"""
Auto-Coder SDK 桥接层

连接 SDK 和底层 auto_coder_runner 功能
"""

import os
import sys
from typing import Any, Dict, Optional, Iterator, Union
from pathlib import Path
import subprocess
from contextlib import contextmanager

from ..exceptions import BridgeError
from ..models.responses import StreamEvent
from autocoder.auto_coder_runner import run_auto_command, configure
from ..models.options import AutoCodeOptions

# 不导入事件类型，使用动态检查避免类型冲突


@contextmanager
def temporary_chdir(path: Union[str, Path]):
    """临时切换工作目录的上下文管理器"""
    original_cwd = os.getcwd()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(original_cwd)


class AutoCoderBridge:
    """桥接层，连接现有功能"""

    def __init__(self, project_root: str, options: AutoCodeOptions):
        """
        初始化桥接层

        Args:
            project_root: 项目根目录
            options: 配置选项
        """
        self.project_root = project_root or os.getcwd()
        self.options = options
        self.allow_chdir = True  # 默认允许切换目录，可以通过配置关闭
        self._setup_environment()

    def _setup_environment(self):
        """设置环境和内存"""
        try:
            # 使用上下文管理器临时切换到项目根目录
            if self.allow_chdir:
                with temporary_chdir(self.project_root):
                    self._init_project_environment()
            else:
                # 不切换目录的情况下初始化
                self._init_project_environment()

        except Exception as e:
            # 设置环境失败不应该阻止程序运行
            pass

    def _init_project_environment(self):
        """初始化项目环境"""
        try:
            self.init_project_if_required()
            from autocoder.auto_coder_runner import start

            start()
        except ImportError:
            # 如果无法导入，跳过初始化
            pass

    def init_project_if_required(self):
        from autocoder.auto_coder_runner import (
            init_project_if_required as init_project_if_required_buildin,
        )

        init_project_if_required_buildin(self.project_root, "*")

    def call_run_auto_command(
        self,
        query: str,
        pre_commit: bool = False,
        pr: bool = False,
        cancel_token: Optional[str] = None,
        extra_args: Optional[Dict[str, Any]] = None,
        stream: bool = True,
    ) -> Iterator[StreamEvent]:
        """
        调用 run_auto_command 功能并返回事件流

        Args:
            query: 查询内容
            pre_commit: 是否预提交
            pr: 是否预提交
            cancel_token: 取消令牌，用于中断执行
            extra_args: 额外参数
            stream: 是否流式返回

        Yields:
            StreamEvent: 事件流

        Raises:
            BridgeError: 桥接层错误
        """
        try:
            # 准备参数
            extra_args = extra_args or {}

            # 根据会话配置确定 conversation_action 和 conversation_id
            from autocoder.common.v2.agent.agentic_edit_types import ConversationAction

            conversation_action = ConversationAction.NEW  # 默认为新对话
            conversation_id = None

            if self.options.session_id:
                # 如果有 session_id，就是恢复特定会话
                conversation_action = ConversationAction.RESUME
                conversation_id = self.options.session_id
            elif self.options.continue_session:
                # 如果是 continue_session，就是继续最近的对话
                conversation_action = ConversationAction.CONTINUE
                conversation_id = None

            # 发送开始事件
            yield StreamEvent(
                event_type="start",
                data={
                    "query": query,
                    "pre_commit": pre_commit,
                    "cancel_token": cancel_token,
                    "conversation_action": conversation_action.value,
                    "conversation_id": conversation_id,
                },
            )

            # 使用上下文管理器切换目录（如果允许）
            context = (
                temporary_chdir(self.project_root)
                if self.allow_chdir
                else _nullcontext()
            )

            with context:
                try:
                    # 实现循环逻辑
                    loop_count = self.options.loop

                    for loop_index in range(loop_count):
                        # 第一次使用原始查询，后续循环使用改进的查询
                        current_query = query
                        if loop_index > 0:
                            # 使用自定义的额外提示或默认提示
                            if self.options.loop_additional_prompt:
                                additional_prompt = self.options.loop_additional_prompt
                            else:
                                additional_prompt = "use git log to get the code changes generated by previous tasks and try to focus on iterative improvements and refinements. Ensure the implementation is complete, functional, and fully usable without any missing features or incomplete functionality. Make sure to use git commit command to make a commit after every single file edit."
                            current_query = f"{query}\n\nAdditional instruction: {additional_prompt}"

                        # 发送循环开始事件
                        if loop_count > 1:
                            yield StreamEvent(
                                event_type="loop_start",
                                data={"content": f"{loop_index + 1}/{loop_count}"},
                            )

                        # 对于循环执行，根据loop_keep_conversation参数决定是否使用CONTINUE模式
                        current_conversation_action = conversation_action
                        current_conversation_id = conversation_id
                        if loop_index > 0 and self.options.loop_keep_conversation:
                            from autocoder.common.v2.agent.agentic_edit_types import (
                                ConversationAction,
                            )

                            current_conversation_action = ConversationAction.CONTINUE
                            current_conversation_id = None

                        events = run_auto_command(
                            query=current_query,
                            pre_commit=pre_commit,
                            pr=pr
                            and (
                                loop_index == loop_count - 1
                            ),  # 只在最后一次循环时创建PR
                            cancel_token=cancel_token,
                            extra_args={
                                **extra_args,
                                "max_turns": self.options.max_turns,
                                "model": self.options.model,
                                "include_rules": self.options.include_rules,
                            },
                            system_prompt=self.options.system_prompt,
                            conversation_action=current_conversation_action,
                            conversation_id=current_conversation_id,
                            is_sub_agent=self.options.is_sub_agent,
                        )

                        # 如果返回的是生成器，逐个处理事件
                        if hasattr(events, "__iter__") and not isinstance(
                            events, (str, bytes)
                        ):
                            for event in events:
                                # 转换事件格式
                                stream_event = self._convert_event_to_stream_event(
                                    event
                                )
                                if stream_event:  # 只yield非None的事件
                                    yield stream_event
                        else:
                            # 如果不是生成器，包装成单个事件
                            yield StreamEvent(
                                event_type="content", data={"content": str(events)}
                            )

                        # 发送循环完成事件
                        if loop_count > 1:
                            yield StreamEvent(
                                event_type="loop_end",
                                data={"content": f"{loop_index + 1}/{loop_count}"},
                            )

                except (ImportError, FileNotFoundError, Exception) as e:
                    raise BridgeError(
                        f"run_auto_command failed: {str(e)}", original_error=e
                    )

            # 发送完成事件
            yield StreamEvent(event_type="end", data={"status": "completed"})

        except Exception as e:
            # 发送错误事件
            yield StreamEvent(
                event_type="error",
                data={"error": str(e), "error_type": type(e).__name__},
            )
            raise BridgeError(f"run_auto_command failed: {str(e)}", original_error=e)

    def _simulate_auto_command_response(self, query: str) -> Iterator[StreamEvent]:
        """
        模拟 auto_command 响应

        Args:
            query: 查询内容

        Yields:
            StreamEvent: 模拟的事件流
        """
        yield StreamEvent(
            event_type="content", data={"content": f"[模拟模式] 正在处理查询: {query}"}
        )
        yield StreamEvent(
            event_type="content",
            data={
                "content": f"[模拟模式] 这是对您查询的模拟响应。在真实环境中，Auto-Coder 会："
            },
        )

        # 根据查询内容生成合适的模拟响应
        if any(
            keyword in query.lower()
            for keyword in ["function", "函数", "write", "create", "写"]
        ):
            yield StreamEvent(event_type="content", data={"content": "1. 分析您的需求"})
            yield StreamEvent(
                event_type="content", data={"content": "2. 生成相应的代码"}
            )
            yield StreamEvent(
                event_type="content", data={"content": "3. 添加适当的注释和文档"}
            )
        elif any(
            keyword in query.lower()
            for keyword in ["error", "错误", "handling", "处理"]
        ):
            yield StreamEvent(
                event_type="content", data={"content": "1. 识别潜在的错误点"}
            )
            yield StreamEvent(
                event_type="content", data={"content": "2. 添加 try-catch 块"}
            )
            yield StreamEvent(
                event_type="content", data={"content": "3. 添加适当的日志记录"}
            )
        elif any(
            keyword in query.lower() for keyword in ["explain", "解释", "how", "如何"]
        ):
            yield StreamEvent(event_type="content", data={"content": "1. 分析代码结构"})
            yield StreamEvent(
                event_type="content", data={"content": "2. 提供详细的解释"}
            )
            yield StreamEvent(event_type="content", data={"content": "3. 给出使用示例"})
        else:
            yield StreamEvent(event_type="content", data={"content": "1. 理解您的请求"})
            yield StreamEvent(
                event_type="content", data={"content": "2. 执行相应的操作"}
            )
            yield StreamEvent(event_type="content", data={"content": "3. 返回结果"})

        yield StreamEvent(
            event_type="content",
            data={
                "content": "\n注意：当前运行在模拟模式下。要使用完整功能，请确保正确安装 Auto-Coder 核心组件。"
            },
        )

    def _convert_event_to_stream_event(self, event: Any) -> Optional[StreamEvent]:
        """
        将 run_auto_command 的事件转换为 StreamEvent

        Args:
            event: 原始事件

        Returns:
            StreamEvent: 转换后的事件，如果不需要转换则返回 None
        """
        try:
            # 获取事件类型名称
            event_class_name = type(event).__name__

            # 根据事件类型名称进行转换
            if event_class_name == "LLMThinkingEvent":
                return StreamEvent(
                    event_type="llm_thinking", data={"text": getattr(event, "text", "")}
                )

            elif event_class_name == "LLMOutputEvent":
                return StreamEvent(
                    event_type="llm_output", data={"text": getattr(event, "text", "")}
                )

            elif event_class_name == "ToolCallEvent":
                tool_name = "Unknown"
                tool_args = {}
                tool_xml = ""

                if hasattr(event, "tool"):
                    tool_name = (
                        type(event.tool).__name__
                        if hasattr(event.tool, "__class__")
                        else "Unknown"
                    )
                    if hasattr(event.tool, "model_dump"):
                        try:
                            tool_args = event.tool.model_dump()
                        except:
                            tool_args = {}
                    elif hasattr(event.tool, "__dict__"):
                        tool_args = event.tool.__dict__

                if hasattr(event, "tool_xml"):
                    tool_xml = event.tool_xml

                return StreamEvent(
                    event_type="tool_call",
                    data={
                        "tool_name": tool_name,
                        "args": tool_args,
                        "tool_xml": tool_xml,
                    },
                )

            elif event_class_name == "ToolResultEvent":
                result_data = {
                    "tool_name": getattr(event, "tool_name", "Unknown"),
                    "success": True,
                    "message": "",
                    "content": None,
                }

                if hasattr(event, "result") and event.result:
                    if hasattr(event.result, "success"):
                        result_data["success"] = event.result.success
                    if hasattr(event.result, "message"):
                        result_data["message"] = event.result.message
                    if hasattr(event.result, "content"):
                        result_data["content"] = event.result.content

                return StreamEvent(event_type="tool_result", data=result_data)

            elif event_class_name == "CompletionEvent":
                result = ""
                if hasattr(event, "completion"):
                    if hasattr(event.completion, "result"):
                        result = event.completion.result
                    elif hasattr(event.completion, "response"):
                        result = event.completion.response

                return StreamEvent(event_type="completion", data={"result": result})

            elif event_class_name == "ErrorEvent":
                return StreamEvent(
                    event_type="error",
                    data={
                        "error": getattr(event, "message", "Unknown error"),
                        "error_type": "AgenticError",
                    },
                )

            elif event_class_name == "TokenUsageEvent":
                usage = getattr(event, "usage", None)
                if usage:
                    # 序列化 SingleOutputMeta 对象
                    usage_data = {
                        "input_tokens_count": getattr(usage, "input_tokens_count", 0),
                        "generated_tokens_count": getattr(
                            usage, "generated_tokens_count", 0
                        ),
                        "reasoning_content": getattr(usage, "reasoning_content", ""),
                        "finish_reason": getattr(usage, "finish_reason", "stop"),
                        "first_token_time": getattr(usage, "first_token_time", 0),
                    }
                else:
                    usage_data = {}

                return StreamEvent(event_type="token_usage", data={"usage": usage_data})

            elif event_class_name == "WindowLengthChangeEvent":
                return StreamEvent(
                    event_type="window_change",
                    data={"tokens_used": getattr(event, "tokens_used", 0)},
                )

            elif event_class_name == "ConversationIdEvent":
                return StreamEvent(
                    event_type="conversation_id",
                    data={"conversation_id": getattr(event, "conversation_id", "")},
                )

            elif event_class_name == "PreCommitEvent":
                # event.commit_result 和 event.tpe 一定存在
                commit_result = event.commit_result
                commit_data = {
                    "success": (
                        commit_result.success
                        if hasattr(commit_result, "success")
                        else False
                    ),
                    "message": (
                        commit_result.message
                        if hasattr(commit_result, "message")
                        else ""
                    ),
                    "commit_hash": (
                        commit_result.commit_hash
                        if hasattr(commit_result, "commit_hash")
                        else ""
                    ),
                    "files_changed": (
                        commit_result.files_changed
                        if hasattr(commit_result, "files_changed")
                        else []
                    ),
                    "error": (
                        commit_result.error if hasattr(commit_result, "error") else None
                    ),
                }

                return StreamEvent(
                    event_type="pre_commit",
                    data={"commit_result": commit_data, "tpe": event.tpe},
                )

            elif event_class_name == "CommitEvent":
                # event.commit_result 和 event.tpe 一定存在
                commit_result = event.commit_result
                commit_data = {
                    "success": (
                        commit_result.success
                        if hasattr(commit_result, "success")
                        else False
                    ),
                    "message": (
                        commit_result.message
                        if hasattr(commit_result, "message")
                        else ""
                    ),
                    "commit_hash": (
                        commit_result.commit_hash
                        if hasattr(commit_result, "commit_hash")
                        else ""
                    ),
                    "files_changed": (
                        commit_result.files_changed
                        if hasattr(commit_result, "files_changed")
                        else []
                    ),
                    "error": (
                        commit_result.error if hasattr(commit_result, "error") else None
                    ),
                }

                return StreamEvent(
                    event_type="commit",
                    data={"commit_result": commit_data, "tpe": event.tpe},
                )

            elif event_class_name == "PullRequestEvent":
                # event.pull_request_result 一定存在
                pr_result = event.pull_request_result
                pr_data = {
                    "success": (
                        pr_result.success if hasattr(pr_result, "success") else False
                    ),
                    "pr_number": (
                        pr_result.pr_number if hasattr(pr_result, "pr_number") else None
                    ),
                    "pr_url": pr_result.pr_url if hasattr(pr_result, "pr_url") else "",
                    "pr_id": pr_result.pr_id if hasattr(pr_result, "pr_id") else None,
                    "error_message": (
                        pr_result.error_message
                        if hasattr(pr_result, "error_message")
                        else None
                    ),
                    "error_code": (
                        pr_result.error_code
                        if hasattr(pr_result, "error_code")
                        else None
                    ),
                    "platform": (
                        pr_result.platform if hasattr(pr_result, "platform") else None
                    ),
                    "raw_response": (
                        pr_result.raw_response
                        if hasattr(pr_result, "raw_response")
                        else None
                    ),
                    "retry_after": (
                        pr_result.retry_after
                        if hasattr(pr_result, "retry_after")
                        else None
                    ),
                }

                return StreamEvent(
                    event_type="pull_request", data={"pull_request_result": pr_data}
                )

            elif event_class_name == "PlanModeRespondEvent":
                result = ""
                if hasattr(event, "completion"):
                    if hasattr(event.completion, "response"):
                        result = event.completion.response

                return StreamEvent(
                    event_type="plan_mode_respond", data={"result": result}
                )

            # 处理简单字符串或其他类型
            elif isinstance(event, str):
                return StreamEvent(event_type="content", data={"content": event})

            # 如果有event_type属性，尝试直接使用
            elif hasattr(event, "event_type"):
                return StreamEvent(
                    event_type=event.event_type,
                    data=getattr(event, "data", {}),
                    timestamp=getattr(event, "timestamp", None),
                )

            # 未知事件类型，包装成内容事件
            else:
                return StreamEvent(event_type="content", data={"content": str(event)})

        except Exception as e:
            # 转换失败，返回错误事件
            return StreamEvent(
                event_type="content",
                data={"content": f"[Event Conversion Error: {str(e)}]"},
            )

    def get_memory(self) -> Dict[str, Any]:
        """
        获取当前内存状态

        Returns:
            Dict[str, Any]: 内存数据
        """
        try:
            if self.allow_chdir:
                with temporary_chdir(self.project_root):
                    return self._get_memory_internal()
            else:
                return self._get_memory_internal()
        except Exception as e:
            raise BridgeError(f"Failed to get memory: {str(e)}", original_error=e)

    def _get_memory_internal(self) -> Dict[str, Any]:
        """内部获取内存实现"""
        try:
            from autocoder.auto_coder_runner import get_memory

            memory_data = get_memory()
            return memory_data
        except ImportError:
            # 模拟内存数据
            return {
                "current_files": {"files": [], "groups": {}},
                "conf": {},
                "exclude_dirs": [],
                "mode": "auto_detect",
            }

    def save_memory(self, memory_data: Dict[str, Any]) -> None:
        """
        保存内存状态

        Args:
            memory_data: 要保存的内存数据
        """
        try:
            if self.allow_chdir:
                with temporary_chdir(self.project_root):
                    self._save_memory_internal(memory_data)
            else:
                self._save_memory_internal(memory_data)
        except Exception as e:
            raise BridgeError(f"Failed to save memory: {str(e)}", original_error=e)

    def _save_memory_internal(self, memory_data: Dict[str, Any]) -> None:
        """内部保存内存实现"""
        try:
            from autocoder.auto_coder_runner import save_memory, memory

            # 更新全局 memory 变量
            memory.update(memory_data)
            save_memory()
        except ImportError:
            # 模拟保存
            pass

    def get_project_config(self) -> Dict[str, Any]:
        """
        获取项目配置

        Returns:
            Dict[str, Any]: 项目配置
        """
        try:
            if self.allow_chdir:
                with temporary_chdir(self.project_root):
                    return self._get_project_config_internal()
            else:
                return self._get_project_config_internal()
        except Exception as e:
            raise BridgeError(
                f"Failed to get project config: {str(e)}", original_error=e
            )

    def _get_project_config_internal(self) -> Dict[str, Any]:
        """内部获取配置实现"""
        try:
            from autocoder.auto_coder_runner import get_final_config

            config = get_final_config()
            return config.__dict__ if hasattr(config, "__dict__") else {}
        except ImportError:
            # 模拟配置
            return {"mock_config": True}

    def setup_project_context(self) -> None:
        """设置项目上下文"""
        try:
            # 确保项目环境已正确初始化
            self._setup_environment()
        except Exception as e:
            raise BridgeError(
                f"Failed to setup project context: {str(e)}", original_error=e
            )

    def cleanup(self) -> None:
        """清理资源"""
        try:
            if self.allow_chdir:
                with temporary_chdir(self.project_root):
                    self._cleanup_internal()
            else:
                self._cleanup_internal()
        except Exception as e:
            # 清理失败不应该阻止程序继续运行，只记录错误
            print(f"Warning: Failed to cleanup resources: {str(e)}")

    def _cleanup_internal(self) -> None:
        """内部清理实现"""
        try:
            from autocoder.auto_coder_runner import stop

            stop()
        except ImportError:
            pass

    def configure_agentic_max_rounds(self, max_rounds: int = 10000) -> None:
        """
        配置代理最大轮数

        Args:
            max_rounds: 最大轮数，默认为10000
        """
        try:
            configure(f"agentic_max_rounds:{max_rounds}", skip_print=True)
        except Exception as e:
            raise BridgeError(
                f"Failed to configure agentic max rounds: {str(e)}", original_error=e
            )

    def configure_model(self, model_name: str) -> None:
        """
        配置模型
        """
        try:
            configure(f"model:{model_name}", skip_print=True)
        except Exception as e:
            raise BridgeError(
                f"Failed to configure model '{model_name}': {str(e)}", original_error=e
            )

    def run_workflow(
        self,
        workflow: str,
        prompt: Optional[str],
        model: Optional[str],
        vars_override: Optional[Dict[str, Any]] = None,
    ) -> Any:
        """
        运行指定的 workflow

        Args:
            workflow: workflow 名称或文件路径
            prompt: 可选的提示内容，会注入到 workflow 的 vars.query 中
            model: 可选的模型名称，覆盖 workflow 配置中的模型
            vars_override: 可选的变量覆盖字典

        Returns:
            WorkflowResult: workflow 执行结果

        Raises:
            BridgeError: 桥接层错误
        """
        try:
            # 使用上下文管理器切换目录（如果允许）
            context = (
                temporary_chdir(self.project_root)
                if self.allow_chdir
                else _nullcontext()
            )

            with context:
                try:
                    from autocoder.auto_coder_runner import (
                        run_workflow as _run_workflow,
                    )

                    return _run_workflow(
                        workflow=workflow,
                        prompt=prompt,
                        model=model,
                        vars_override=vars_override,
                    )
                except (ImportError, FileNotFoundError, Exception) as e:
                    raise BridgeError(
                        f"run_workflow failed: {str(e)}", original_error=e
                    )

        except Exception as e:
            raise BridgeError(f"run_workflow failed: {str(e)}", original_error=e)

    def __enter__(self):
        """上下文管理器入口"""
        self.setup_project_context()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口"""
        self.cleanup()


class _nullcontext:
    """Python 3.7 兼容的 nullcontext"""

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return False



"""
Module for compiling/checking ReactJS code.
This module provides functionality to compile/check ReactJS code using tools like npm, babel, and webpack.
"""

import os
import json
import subprocess
import tempfile
import time
import re
from typing import Dict, List, Any, Optional, Tuple

from autocoder.compilers.base_compiler import BaseCompiler
from autocoder.compilers.models import (
    CompilationError, 
    FileCompilationResult, 
    ProjectCompilationResult,
    CompilationErrorPosition,
    CompilationErrorSeverity
)

class ReactJSCompiler(BaseCompiler):
    """
    A class that provides compilation/checking functionality for ReactJS code.
    """
    
    def __init__(self, verbose: bool = False):
        """
        Initialize the ReactJSCompiler.
        
        Args:
            verbose (bool): Whether to display verbose output.
        """
        super().__init__(verbose)
    
    def get_supported_extensions(self) -> List[str]:
        """
        Get the list of file extensions supported by this compiler.
        
        Returns:
            List[str]: List of supported file extensions.
        """
        return ['.js', '.jsx', '.ts', '.tsx']
    
    def _check_dependencies(self) -> bool:
        """
        Check if required dependencies (node, npm) are installed.
        
        Returns:
            bool: True if all dependencies are available, False otherwise.
        """
        try:
            # Check if node is installed
            node_process = subprocess.run(
                ['node', '--version'], 
                stdout=subprocess.PIPE, 
                stderr=subprocess.PIPE
            )
            
            # Check if npm is installed
            npm_process = subprocess.run(
                ['npm', '--version'], 
                stdout=subprocess.PIPE, 
                stderr=subprocess.PIPE
            )
            
            return node_process.returncode == 0 and npm_process.returncode == 0
        except (subprocess.SubprocessError, FileNotFoundError):
            return False
    
    def _is_react_project(self, project_path: str) -> bool:
        """
        Check if a directory is a React project.
        
        Args:
            project_path (str): Path to check.
            
        Returns:
            bool: True if the directory contains a React project, False otherwise.
        """
        # Check for package.json
        package_json_path = os.path.join(project_path, 'package.json')
        if not os.path.exists(package_json_path):
            return False
        
        try:
            # Read package.json to check for React dependency
            with open(package_json_path, 'r') as f:
                package_data = json.load(f)
            
            dependencies = package_data.get('dependencies', {})
            dev_dependencies = package_data.get('devDependencies', {})
            
            # Check if React is a dependency
            return 'react' in dependencies or 'react' in dev_dependencies
        except:
            return False
    
    def _parse_error_position(self, error_text: str) -> Tuple[int, Optional[int]]:
        """
        Parse line and column from error text.
        
        Args:
            error_text (str): Error text to parse.
            
        Returns:
            Tuple[int, Optional[int]]: Line and column (if found).
        """
        # Look for common patterns like "Line X:Y" or "line X, column Y" or "LX:CY"
        patterns = [
            r'[Ll]ine\s+(\d+)(?:[,:]\s*(?:column\s+)?(\d+))?',
            r'[Ll](\d+)(?:[,:]\s*[Cc](\d+))?',
            r'[\(\[](\d+)[,:]\s*(\d+)[\)\]]'
        ]
        
        for pattern in patterns:
            match = re.search(pattern, error_text)
            if match:
                line = int(match.group(1))
                column = int(match.group(2)) if match.group(2) else None
                return line, column
        
        return 1, None  # Default to line 1 if not found
    
    def _parse_compilation_error(self, error_text: str, file_path: str) -> CompilationError:
        """
        Parse a compilation error message and create a CompilationError object.
        
        Args:
            error_text (str): Raw error message.
            file_path (str): Path to the file with the error.
            
        Returns:
            CompilationError: Parsed error object.
        """
        line, column = self._parse_error_position(error_text)
        
        return CompilationError(
            message=error_text,
            severity=CompilationErrorSeverity.ERROR,
            position=CompilationErrorPosition(
                line=line,
                column=column
            ),
            file_path=file_path,
            code="react-compilation-error"
        )
    
    def _check_file_syntax(self, file_path: str) -> Dict[str, Any]:
        """
        Check syntax of a React/JavaScript file using Babel.
        
        Args:
            file_path (str): Path to the file to check.
            
        Returns:
            Dict[str, Any]: Dictionary containing syntax check results.
        """
        errors = []
        error_count = 0
        warning_count = 0
        
        try:
            # Create a temporary directory
            with tempfile.TemporaryDirectory() as temp_dir:
                # Create a minimal package.json
                package_json = {
                    "name": "syntax-checker",
                    "version": "1.0.0",
                    "description": "Temporary package for syntax checking",
                    "dependencies": {
                        "@babel/core": "^7.14.0",
                        "@babel/preset-env": "^7.14.0",
                        "@babel/preset-react": "^7.13.13"
                    },
                    "scripts": {
                        "check": "babel --presets=@babel/preset-env,@babel/preset-react --no-babelrc --filename \"$FILE\" --check-syntax-only"
                    }
                }
                
                # If the file is TypeScript, add TypeScript dependencies
                if file_path.endswith('.ts') or file_path.endswith('.tsx'):
                    package_json["dependencies"]["@babel/preset-typescript"] = "^7.13.0"
                    package_json["scripts"]["check"] = "babel --presets=@babel/preset-env,@babel/preset-react,@babel/preset-typescript --no-babelrc --filename \"$FILE\" --check-syntax-only"
                
                # Write package.json
                with open(os.path.join(temp_dir, 'package.json'), 'w') as f:
                    json.dump(package_json, f)
                
                # Install dependencies
                subprocess.run(
                    ['npm', 'install', '--silent'], 
                    cwd=temp_dir,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE
                )
                
                # Run babel to check syntax
                cmd = f'FILE={file_path} npm run check -- {file_path}'
                
                process = subprocess.run(
                    cmd,
                    shell=True,
                    cwd=temp_dir,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True
                )
                
                # Check for syntax errors
                if process.returncode != 0:
                    for line in process.stderr.splitlines():
                        if line and not line.startswith('npm'):
                            error = self._parse_compilation_error(line, file_path)
                            errors.append(error)
                            error_count += 1
        
        except Exception as e:
            errors.append(CompilationError(
                message=f"Error checking syntax: {str(e)}",
                severity=CompilationErrorSeverity.ERROR,
                position=CompilationErrorPosition(line=1),
                file_path=file_path,
                code="syntax-check-error"
            ))
            error_count += 1
        
        return {
            'success': error_count == 0,
            'errors': errors,
            'error_count': error_count,
            'warning_count': warning_count
        }
    
    def compile_file(self, file_path: str) -> Dict[str, Any]:
        """
        Compile (check) a single ReactJS file.
        
        Args:
            file_path (str): Path to the file to compile.
            
        Returns:
            Dict[str, Any]: Compilation results.
        """
        if not os.path.exists(file_path):
            return FileCompilationResult(
                file_path=file_path,
                success=False,
                language="react",
                error_message=f"File not found: {file_path}"
            ).model_dump()
        
        if not self.is_supported_file(file_path):
            return FileCompilationResult(
                file_path=file_path,
                success=False,
                language="react",
                error_message=f"Unsupported file type: {file_path}"
            ).model_dump()
        
        if not self._check_dependencies():
            return FileCompilationResult(
                file_path=file_path,
                success=False,
                language="react",
                error_message="Node.js and npm are required but not found. Please make sure they are installed."
            ).model_dump()
        
        start_time = time.time()
        
        # Check syntax
        syntax_result = self._check_file_syntax(file_path)
        
        # Calculate execution time
        execution_time_ms = int((time.time() - start_time) * 1000)
        
        # Create the result
        result = FileCompilationResult(
            file_path=file_path,
            success=syntax_result['success'],
            language="react",
            errors=syntax_result['errors'],
            error_count=syntax_result['error_count'],
            warning_count=syntax_result['warning_count'],
            info_count=0,
            execution_time_ms=execution_time_ms
        )
        
        return result.model_dump()
    
    def compile_project(self, project_path: str) -> Dict[str, Any]:
        """
        Compile (build) a ReactJS project.
        
        Args:
            project_path (str): Path to the project directory.
            
        Returns:
            Dict[str, Any]: Compilation results.
        """
        if not os.path.exists(project_path):
            return ProjectCompilationResult(
                project_path=project_path,
                success=False,
                total_files=0,
                error_message=f"Project directory not found: {project_path}"
            ).model_dump()
        
        if not self._check_dependencies():
            return ProjectCompilationResult(
                project_path=project_path,
                success=False,
                total_files=0,
                error_message="Node.js and npm are required but not found. Please make sure they are installed."
            ).model_dump()
        
        if not self._is_react_project(project_path):
            return ProjectCompilationResult(
                project_path=project_path,
                success=False,
                total_files=0,
                error_message=f"The directory does not appear to be a React project: {project_path}"
            ).model_dump()
        
        start_time = time.time()
        
        # Run npm build
        cmd = ['npm', 'run', 'build']
        
        process = subprocess.run(
            cmd,
            cwd=project_path,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        
        success = process.returncode == 0
        output = process.stdout + process.stderr
        errors = []
        error_count = 0
        warning_count = 0
        file_results = {}
        total_files = 0
        files_with_errors = 0
        
        # Find React files in the project
        react_files = []
        for root, _, files in os.walk(project_path):
            for file in files:
                if any(file.endswith(ext) for ext in self.get_supported_extensions()):
                    file_path = os.path.join(root, file)
                    react_files.append(file_path)
                    total_files += 1
        
        if not success:
            # Parse build errors
            file_error_map = {}  # Maps file paths to errors
            
            # Look for webpack error patterns
            error_blocks = re.split(r'Module build failed|Failed to compile', output)
            
            if len(error_blocks) > 1:  # If we found error blocks
                for block in error_blocks[1:]:  # Skip the first block (before the error)
                    # Try to find file path
                    file_match = re.search(r'((?:\.\./)*(?:src|components|pages|views).*?\.(?:js|jsx|ts|tsx))', block)
                    if file_match:
                        error_file = os.path.join(project_path, file_match.group(1))
                        if os.path.exists(error_file):
                            error_message = block.strip()
                            
                            if error_file not in file_error_map:
                                file_error_map[error_file] = []
                            
                            error = self._parse_compilation_error(error_message, error_file)
                            file_error_map[error_file].append(error)
                            errors.append(error)
                            error_count += 1
            
            # If no structured errors found, fall back to checking each file individually
            if len(errors) == 0:
                for file_path in react_files:
                    file_result = self.compile_file(file_path)
                    file_results[file_path] = file_result
                    
                    if not file_result['success']:
                        files_with_errors += 1
                    
                    error_count += file_result['error_count']
                    warning_count += file_result['warning_count']
                    
                    # Add file errors to the project errors list
                    for error in file_result.get('errors', []):
                        errors.append(error)
            else:
                # Create file results from the error map
                for file_path in react_files:
                    file_errors = file_error_map.get(file_path, [])
                    
                    file_result = FileCompilationResult(
                        file_path=file_path,
                        success=len(file_errors) == 0,
                        language="react",
                        errors=file_errors,
                        error_count=len(file_errors),
                        warning_count=0,
                        info_count=0
                    )
                    
                    file_results[file_path] = file_result.model_dump()
                    
                    if len(file_errors) > 0:
                        files_with_errors += 1
        else:
            # If build succeeded, create a success result for each file
            for file_path in react_files:
                file_result = FileCompilationResult(
                    file_path=file_path,
                    success=True,
                    language="react",
                    errors=[],
                    error_count=0,
                    warning_count=0,
                    info_count=0
                )
                
                file_results[file_path] = file_result.model_dump()
        
        # Calculate execution time
        execution_time_ms = int((time.time() - start_time) * 1000)
        
        # Create the project result
        result = ProjectCompilationResult(
            project_path=project_path,
            success=success,
            total_files=total_files,
            files_with_errors=files_with_errors,
            total_errors=error_count,
            total_warnings=warning_count,
            total_infos=0,
            file_results={p: FileCompilationResult(**r) for p, r in file_results.items()},
            output_directory=os.path.join(project_path, 'build') if success else None
        )
        
        return result.model_dump()
    
    def format_compile_result(self, compile_result: Dict[str, Any]) -> str:
        """
        Format compilation results into a human-readable string.
        
        Args:
            compile_result (Dict[str, Any]): The compilation result dictionary.
            
        Returns:
            str: A formatted string representation of the compilation results.
        """
        if 'project_path' in compile_result:
            # This is a project result
            return ProjectCompilationResult(**compile_result).to_str()
        else:
            # This is a file result
            return FileCompilationResult(**compile_result).to_str()

def compile_reactjs_file(file_path: str, verbose: bool = False) -> Dict[str, Any]:
    """
    Utility function to compile a single ReactJS file.
    
    Args:
        file_path (str): Path to the file to compile.
        verbose (bool): Whether to display verbose output.
        
    Returns:
        Dict[str, Any]: A dictionary containing compilation results.
    """
    compiler = ReactJSCompiler(verbose=verbose)
    return compiler.compile_file(file_path)

def compile_reactjs_project(project_path: str, verbose: bool = False) -> Dict[str, Any]:
    """
    Utility function to compile a ReactJS project.
    
    Args:
        project_path (str): Path to the project directory.
        verbose (bool): Whether to display verbose output.
        
    Returns:
        Dict[str, Any]: A dictionary containing compilation results.
    """
    compiler = ReactJSCompiler(verbose=verbose)
    return compiler.compile_project(project_path) 



import os
import json
from loguru import logger

def load_failed_files(failed_files_path: str) -> set:
    """
    Load the set of failed file paths from a JSON file.
    
    Args:
        failed_files_path: Path to the JSON file storing failed files.
    
    Returns:
        A set of failed file paths.
    """
    directory = os.path.dirname(failed_files_path)
    if not os.path.exists(directory):
        os.makedirs(directory, exist_ok=True)
    if os.path.exists(failed_files_path):
        try:
            with open(failed_files_path, "r", encoding="utf-8") as f:
                return set(json.load(f))
        except Exception:
            return set()
    return set()

def save_failed_files(failed_files_path: str, failed_files: set) -> None:
    """
    Save the set of failed file paths to a JSON file.
    
    Args:
        failed_files_path: Path to the JSON file.
        failed_files: A set of failed file paths.
    """
    try:
        with open(failed_files_path, "w", encoding="utf-8") as f:
            json.dump(list(failed_files), f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving failed files list: {e}")






"""
Tools Manager Usage Examples

工具管理器的使用示例和演示代码。
"""

import os
import tempfile
from pathlib import Path
from autocoder.common.tools_manager import ToolsManager


def example_basic_usage():
    """基本使用示例"""
    print("=== 基本使用示例 ===")
    
    # 创建工具管理器实例
    manager = ToolsManager()
    
    # 加载工具
    result = manager.load_tools()
    
    print(f"加载结果: {'成功' if result.success else '失败'}")
    print(f"工具总数: {result.total_count}")
    print(f"失败数量: {result.failed_count}")
    
    if result.success:
        print("\n工具列表:")
        for tool in result.tools:
            print(f"- {tool.name}: {tool.path}")
    
    print()


def example_get_tools_prompt():
    """获取工具prompt示例"""
    print("=== 获取工具Prompt示例 ===")
    
    manager = ToolsManager()
    
    # 获取工具prompt
    prompt = manager.get_tools_prompt.prompt()
    print("生成的Prompt:")
    print(prompt)
    print()


def example_find_specific_tool():
    """查找特定工具示例"""
    print("=== 查找特定工具示例 ===")
    
    manager = ToolsManager()
    
    # 获取所有工具名称
    tool_names = manager.list_tool_names()
    print(f"所有工具名称: {tool_names}")
    
    # 查找特定工具
    if tool_names:
        first_tool_name = tool_names[0]
        tool = manager.get_tool_by_name(first_tool_name)
        if tool:
            print(f"\n找到工具 '{first_tool_name}':")
            print(f"  路径: {tool.path}")
            print(f"  可执行: {tool.is_executable}")
            print(f"  帮助信息: {tool.help_text[:100]}...")
    
    print()


def example_create_test_tools():
    """创建测试工具示例"""
    print("=== 创建测试工具示例 ===")
    
    # 创建临时目录作为工具目录
    with tempfile.TemporaryDirectory() as temp_dir:
        tools_dir = Path(temp_dir) / ".autocodertools"
        tools_dir.mkdir(parents=True)
        
        # 创建一个Python工具
        python_tool = tools_dir / "test_tool.py"
        python_tool.write_text("""#!/usr/bin/env python3
# 描述: 这是一个测试工具
# 用法: test_tool.py [选项]

import sys

def main():
    if len(sys.argv) > 1 and sys.argv[1] in ['help', '-h', '--help']:
        print("测试工具使用说明:")
        print("  test_tool.py help    - 显示此帮助信息")
        print("  test_tool.py run     - 运行测试")
        return
    
    print("测试工具正在运行...")

if __name__ == "__main__":
    main()
""")
        
        # 设置执行权限
        python_tool.chmod(0o755)
        
        # 创建一个Shell工具
        shell_tool = tools_dir / "deploy.sh"
        shell_tool.write_text("""#!/bin/bash
# 描述: 部署脚本
# 用法: deploy.sh [环境]

if [ "$1" = "help" ] || [ "$1" = "-h" ]; then
    echo "部署脚本使用说明:"
    echo "  deploy.sh help       - 显示此帮助信息"
    echo "  deploy.sh dev        - 部署到开发环境"
    echo "  deploy.sh prod       - 部署到生产环境"
    exit 0
fi

echo "开始部署到环境: ${1:-dev}"
""")
        
        shell_tool.chmod(0o755)
        
        # 使用自定义目录创建工具管理器
        manager = ToolsManager(tools_dirs=[str(tools_dir)])
        
        # 加载工具
        result = manager.load_tools(force_reload=True)
        
        print(f"测试工具加载结果: {'成功' if result.success else '失败'}")
        print(f"找到工具数量: {len(result.tools)}")
        
        for tool in result.tools:
            print(f"\n工具: {tool.name}")
            print(f"  路径: {tool.path}")
            print(f"  扩展名: {tool.file_extension}")
            print(f"  可执行: {tool.is_executable}")
            print(f"  帮助信息:\n{tool.help_text}")
    
    print()


def main():
    """运行所有示例"""
    print("工具管理器使用示例")
    print("=" * 50)
    
    example_basic_usage()
    example_get_tools_prompt()
    example_find_specific_tool()
    example_create_test_tools()
    
    print("示例运行完成!")


if __name__ == "__main__":
    main()






import time
import json
from rich.console import Console
from rich.panel import Panel
from autocoder.common.async_prompt import prompt
from prompt_toolkit.formatted_text import FormattedText
from loguru import logger
import byzerllm
from typing import Optional
from autocoder.common.auto_coder_lang import get_message
from autocoder.common.memory_manager import save_to_memory_file
from autocoder.common.utils_code_auto_generate import stream_chat_with_continue
from autocoder.utils.auto_coder_utils.chat_stream_out import stream_out
from autocoder.common.printer import Printer
from autocoder.common.tokens import count_string_tokens as count_tokens
from autocoder.privacy.model_filter import ModelPathFilter
from autocoder.common.result_manager import ResultManager
from autocoder.events.event_manager_singleton import get_event_manager
from autocoder.events import event_content as EventContentCreator
from autocoder.events.event_types import EventMetadata
from autocoder.common.mcp_tools.server import get_mcp_server
from autocoder.common.mcp_tools.types import McpRequest
from autocoder.utils.llms import get_llm_names
from autocoder.run_context import get_run_context, RunMode
from autocoder.common.action_yml_file_manager import ActionYmlFileManager
from autocoder.common.conversations.get_conversation_manager import (
    get_conversation_manager,
)


class ChatAgent:
    def __init__(self, args, llm, raw_args):
        self.args = args
        self.llm = llm
        self.raw_args = raw_args
        self.console = Console()
        self.result_manager = ResultManager()
        self.conversation_manager = get_conversation_manager()
        # 生成命名空间用于会话隔离
        self.namespace = self._generate_namespace()

    def _generate_namespace(self) -> Optional[str]:
        """
        生成命名空间，用于会话隔离

        Returns:
            str: 基于项目路径的命名空间
        """
        return None

    def run(self):
        """执行 chat 命令的主要逻辑"""
        # 统一格式
        # {"command1": {"args": ["arg1", "arg2"], "kwargs": {"key1": "value1", "key2": "value2"}}}
        if isinstance(self.args.action, dict):
            commands_info = self.args.action
        else:
            commands_info = {}
            for command in self.args.action:
                commands_info[command] = {}

        # 处理新会话
        if self.args.new_session:
            self._handle_new_session()
            if (
                not self.args.query
                or (
                    self.args.query_prefix and self.args.query == self.args.query_prefix
                )
                or (
                    self.args.query_suffix and self.args.query == self.args.query_suffix
                )
            ):
                return

        # 确保有当前会话，如果没有则创建
        current_conversation_id = self.conversation_manager.get_current_conversation_id(
            self.namespace
        )
        if not current_conversation_id:
            current_conversation_id = self.conversation_manager.create_conversation(
                name="Chat Session", description="Auto-coder chat session"
            )
            self.conversation_manager.set_current_conversation(
                current_conversation_id, self.namespace
            )

        # 添加用户消息到当前会话
        self.conversation_manager.append_message_to_current(
            role="user", content=self.args.query, namespace=self.namespace
        )

        # 获取聊天模型
        if self.llm.get_sub_client("chat_model"):
            chat_llm = self.llm.get_sub_client("chat_model")
        else:
            chat_llm = self.llm

        # 构建对话上下文
        loaded_conversations = self._build_conversations(commands_info)

        # 处理人工模型模式
        if get_run_context().mode != RunMode.WEB and self.args.human_as_model:
            return self._handle_human_as_model(loaded_conversations, commands_info)

        # 计算耗时
        start_time = time.time()
        commit_file_name = None

        # 根据命令类型处理不同的响应
        v = self._get_response(commands_info, loaded_conversations, chat_llm)
        if isinstance(v, tuple):
            v, commit_file_name = v

        # 输出响应
        model_name = ",".join(get_llm_names(chat_llm))
        assistant_response, last_meta = stream_out(
            v,
            request_id=self.args.request_id,
            console=self.console,
            model_name=model_name,
            args=self.args,
        )

        self.result_manager.append(
            content=assistant_response,
            meta={"action": "chat", "input": {"query": self.args.query}},
        )

        # 处理学习命令的特殊逻辑
        if "learn" in commands_info and commit_file_name:
            self._handle_learn_command(commit_file_name, assistant_response)

        # 打印统计信息
        if last_meta:
            self._print_stats(last_meta, start_time, model_name)

        # 更新聊天历史 - 添加助手回复到当前会话
        self.conversation_manager.append_message_to_current(
            role="assistant", content=assistant_response, namespace=self.namespace
        )

        # 处理后续命令
        self._handle_post_commands(commands_info, assistant_response)

    def _handle_new_session(self):
        """处理新会话逻辑"""
        # 创建新的对话会话
        conversation_id = self.conversation_manager.create_conversation(
            name="New Chat Session",
            description=f"Chat session started at {time.strftime('%Y-%m-%d %H:%M:%S')}",
        )

        # 设置为当前会话
        self.conversation_manager.set_current_conversation(
            conversation_id, self.namespace
        )

        self.result_manager.add_result(
            content=get_message("new_session_started"),
            meta={"action": "chat", "input": {"query": self.args.query}},
        )
        self.console.print(
            Panel(
                get_message("new_session_started"),
                title="Session Status",
                expand=False,
                border_style="green",
            )
        )

    def _get_current_conversation_messages(self):
        """获取当前会话的消息列表"""
        current_conversation_id = self.conversation_manager.get_current_conversation_id(
            self.namespace
        )
        if current_conversation_id:
            messages = self.conversation_manager.get_messages(current_conversation_id)
            result = []
            for msg in messages:
                msg_dict = {"role": msg["role"], "content": msg["content"]}
                # 保留 reasoning_content（若存在）
                if msg.get("reasoning_content"):
                    msg_dict["reasoning_content"] = msg["reasoning_content"]
                result.append(msg_dict)
            return result
        return []

    def _build_conversations(self, commands_info):
        """构建对话上下文"""
        source_count = 0
        pre_conversations = []
        context_content = self.args.context if self.args.context else ""

        if self.args.context:
            try:
                context = json.loads(self.args.context)
                if "file_content" in context:
                    context_content = context["file_content"]
            except:
                pass

            pre_conversations.append(
                {
                    "role": "user",
                    "content": f"请阅读下面的代码和文档：\n\n <files>\n{context_content}\n</files>",
                },
            )
            pre_conversations.append({"role": "assistant", "content": "read"})
            source_count += 1

        # 构建索引和过滤文件
        if "no_context" not in commands_info:
            from autocoder.index.entry import build_index_and_filter_files
            from autocoder.pyproject import PyProject
            from autocoder.tsproject import TSProject
            from autocoder.suffixproject import SuffixProject
            from autocoder.default_project import DefaultProject

            if self.args.project_type == "ts":
                pp = TSProject(args=self.args, llm=self.llm)
            elif self.args.project_type == "py":
                pp = PyProject(args=self.args, llm=self.llm)
            elif not self.args.project_type or self.args.project_type == "*":
                pp = DefaultProject(args=self.args, llm=self.llm)
            else:
                pp = SuffixProject(args=self.args, llm=self.llm, file_filter=None)
            pp.run()
            sources = pp.sources

            # 应用模型过滤器
            chat_llm = self.llm.get_sub_client("chat_model") or self.llm
            model_filter = ModelPathFilter.from_model_object(chat_llm, self.args)
            filtered_sources = []
            printer = Printer()
            for source in sources:
                if model_filter.is_accessible(source.module_name):
                    filtered_sources.append(source)
                else:
                    printer.print_in_terminal(
                        "index_file_filtered",
                        style="yellow",
                        file_path=source.module_name,
                        model_name=",".join(get_llm_names(chat_llm)),
                    )

            s = build_index_and_filter_files(
                llm=self.llm, args=self.args, sources=filtered_sources
            ).to_str()

            if s:
                pre_conversations.append(
                    {
                        "role": "user",
                        "content": f"请阅读下面的代码和文档：\n\n <files>\n{s}\n</files>",
                    }
                )
                pre_conversations.append({"role": "assistant", "content": "read"})
                source_count += 1

        # 获取当前会话的历史消息
        current_messages = self._get_current_conversation_messages()
        loaded_conversations = pre_conversations + current_messages
        return loaded_conversations

    def _handle_human_as_model(self, loaded_conversations, commands_info):
        """处理人工模型模式"""

        @byzerllm.prompt()
        def chat_with_human_as_model(
            source_codes, pre_conversations, last_conversation
        ):
            """
            {% if source_codes %}
            {{ source_codes }}
            {% endif %}

            {% if pre_conversations %}
            下面是我们之间的历史对话，假设我是A，你是B。
            <conversations>
            {% for conv in pre_conversations %}
            {{ "A" if conv.role == "user" else "B" }}: {{ conv.content }}
            {% endfor %}
            </conversations>
            {% endif %}


            参考上面的文件以及历史对话，回答用户的问题。
            用户的问题: {{ last_conversation.content }}
            """

        source_count = 0
        if self.args.context:
            source_count += 1
        if "no_context" not in commands_info:
            source_count += 1

        source_codes_conversations = loaded_conversations[0 : source_count * 2]
        source_codes = ""
        for conv in source_codes_conversations:
            if conv["role"] == "user":
                source_codes += conv["content"]

        chat_content = chat_with_human_as_model.prompt(
            source_codes=source_codes,
            pre_conversations=loaded_conversations[source_count * 2 : -1],
            last_conversation=loaded_conversations[-1],
        )

        with open(self.args.target_file, "w", encoding="utf-8") as f:
            f.write(chat_content)

        try:
            import pyperclip

            pyperclip.copy(chat_content)
            self.console.print(
                Panel(
                    get_message("chat_human_as_model_instructions"),
                    title="Instructions",
                    border_style="blue",
                    expand=False,
                )
            )
        except Exception:
            logger.warning(get_message("clipboard_not_supported"))
            self.console.print(
                Panel(
                    get_message("human_as_model_instructions_no_clipboard"),
                    title="Instructions",
                    border_style="blue",
                    expand=False,
                )
            )

        lines = []
        while True:
            line = prompt(FormattedText([("#00FF00", "> ")]), multiline=False)
            line_lower = line.strip().lower()
            if line_lower in ["eof", "/eof"]:
                break
            elif line_lower in ["/clear"]:
                lines = []
                print("\033[2J\033[H")  # Clear terminal screen
                continue
            elif line_lower in ["/break"]:
                raise Exception("User requested to break the operation.")
            lines.append(line)

        result = "\n".join(lines)

        self.result_manager.append(
            content=result, meta={"action": "chat", "input": {"query": self.args.query}}
        )

        # 更新当前会话的历史记录
        self.conversation_manager.append_message_to_current(
            role="assistant", content=result, namespace=self.namespace
        )

        if "save" in commands_info:
            # 获取当前会话消息用于保存
            current_messages = self._get_current_conversation_messages()
            save_to_memory_file(
                ask_conversation=current_messages,
                query=self.args.query,
                response=result,
            )
            printer = Printer()
            printer.print_in_terminal("memory_save_success")
        return {}

    def _get_response(self, commands_info, loaded_conversations, chat_llm):
        """根据命令类型获取响应"""
        if "rag" in commands_info:
            from autocoder.rag.rag_entry import RAGFactory

            self.args.enable_rag_search = True
            self.args.enable_rag_context = False
            rag = RAGFactory.get_rag(llm=chat_llm, args=self.args, path="")
            response = rag.stream_chat_oai(conversations=loaded_conversations)[0]
            return (item for item in response)

        elif "mcp" in commands_info:
            mcp_server = get_mcp_server()
            pos_args = commands_info["mcp"].get("args", [])
            final_query = pos_args[0] if pos_args else self.args.query
            response = mcp_server.send_request(
                McpRequest(
                    query=final_query,
                    model=self.args.inference_model or self.args.model,
                    product_mode=self.args.product_mode,
                )
            )
            return [[response.result, None]]

        elif "review" in commands_info:
            from autocoder.agent.auto_review_commit import AutoReviewCommit

            reviewer = AutoReviewCommit(llm=chat_llm, args=self.args)
            pos_args = commands_info["review"].get("args", [])
            final_query = pos_args[0] if pos_args else self.args.query
            kwargs = commands_info["review"].get("kwargs", {})
            commit_id = kwargs.get("commit", None)
            return reviewer.review_commit(
                query=final_query,
                conversations=loaded_conversations,
                commit_id=commit_id,
            )

        elif "learn" in commands_info:
            from autocoder.agent.auto_learn_from_commit import AutoLearnFromCommit

            learner = AutoLearnFromCommit(llm=chat_llm, args=self.args)
            pos_args = commands_info["learn"].get("args", [])
            final_query = pos_args[0] if pos_args else self.args.query
            return learner.learn_from_commit(
                query=final_query, conversations=loaded_conversations
            )

        else:
            # 预估token数量
            dumped_conversations = json.dumps(loaded_conversations, ensure_ascii=False)
            estimated_input_tokens = count_tokens(dumped_conversations)
            printer = Printer()
            printer.print_in_terminal(
                "estimated_chat_input_tokens",
                style="yellow",
                estimated_input_tokens=estimated_input_tokens,
            )

            return stream_chat_with_continue(
                llm=chat_llm,
                conversations=loaded_conversations,
                llm_config={},
                args=self.args,
            )

    def _handle_learn_command(self, commit_file_name, assistant_response):
        """处理学习命令的特殊逻辑"""
        if commit_file_name:
            # 使用 ActionYmlFileManager 更新 YAML 文件
            action_manager = ActionYmlFileManager(self.args.source_dir)
            if not action_manager.update_yaml_field(
                commit_file_name, "how_to_reproduce", assistant_response
            ):
                printer = Printer()
                printer.print_in_terminal(
                    "yaml_save_error", style="red", yaml_file=commit_file_name
                )

    def _print_stats(self, last_meta, start_time, model_name):
        """打印统计信息"""
        elapsed_time = time.time() - start_time
        printer = Printer()
        speed = last_meta.generated_tokens_count / elapsed_time

        # Get model info for pricing
        from autocoder.utils import llms as llm_utils

        model_info = llm_utils.get_model_info(model_name, self.args.product_mode) or {}
        input_price = model_info.get("input_price", 0.0) if model_info else 0.0
        output_price = model_info.get("output_price", 0.0) if model_info else 0.0

        # Calculate costs
        input_cost = (
            last_meta.input_tokens_count * input_price
        ) / 1000000  # Convert to millions
        output_cost = (
            last_meta.generated_tokens_count * output_price
        ) / 1000000  # Convert to millions

        printer.print_in_terminal(
            "stream_out_stats",
            model_name=model_name,
            elapsed_time=elapsed_time,
            first_token_time=last_meta.first_token_time,
            input_tokens=last_meta.input_tokens_count,
            output_tokens=last_meta.generated_tokens_count,
            input_cost=round(input_cost, 4),
            output_cost=round(output_cost, 4),
            speed=round(speed, 2),
        )

        get_event_manager(self.args.event_file).write_result(
            EventContentCreator.create_result(
                content=EventContentCreator.ResultTokenStatContent(
                    model_name=model_name,
                    elapsed_time=elapsed_time,
                    input_tokens=last_meta.input_tokens_count,
                    output_tokens=last_meta.generated_tokens_count,
                    input_cost=round(input_cost, 4),
                    output_cost=round(output_cost, 4),
                    speed=round(speed, 2),
                )
            ).to_dict(),
            metadata=EventMetadata(action_file=self.args.file).to_dict(),
        )

    def _handle_post_commands(self, commands_info, assistant_response):
        """处理后续命令"""
        if "copy" in commands_info:
            # copy assistant_response to clipboard
            import pyperclip

            try:
                pyperclip.copy(assistant_response)
            except:
                print(
                    "pyperclip not installed or clipboard is not supported, instruction will not be copied to clipboard."
                )

        if "save" in commands_info:
            # 获取当前会话消息用于保存
            current_messages = self._get_current_conversation_messages()
            tmp_dir = save_to_memory_file(
                ask_conversation=current_messages,
                query=self.args.query,
                response=assistant_response,
            )
            printer = Printer()
            printer.print_in_terminal(
                "memory_save_success", style="green", path=tmp_dir
            )

            if len(commands_info["save"]["args"]) > 0:
                # 保存到指定文件
                with open(commands_info["save"]["args"][0], "w", encoding="utf-8") as f:
                    f.write(assistant_response)



"""
Tool Call Plugins

工具调用插件系统，提供插件接口和基础实现。
"""

from .plugin_interface import ToolCallPlugin, PluginPriority
from .base_plugin import BaseToolCallPlugin

__all__ = [
    "ToolCallPlugin",
    "BaseToolCallPlugin", 
    "PluginPriority"
] 


"""
优先级目录查找器模块

一个灵活的目录查找工具，支持多种查找策略和验证规则，
用于在多个优先级目录中查找有效的目录。

主要功能：
- 多种查找策略：顺序查找、合并查找、全部查找
- 可配置的验证规则：文件类型、数量等
- 灵活的路径配置：支持环境变量和用户主目录展开
- 标准路径模式：项目级、项目.auto-coder、全局目录
"""

from .models import (
    SearchStrategy,
    ValidationMode,
    FileTypeFilter,
    DirectoryConfig,
    SearchResult,
    FinderConfig
)

from .finder import (
    PriorityDirectoryFinder,
    QuickFinder
)

# 版本信息
__version__ = "1.0.0"

# 导出的主要类和函数
__all__ = [
    # 枚举类型
    "SearchStrategy",
    "ValidationMode",
    
    # 数据模型
    "FileTypeFilter",
    "DirectoryConfig", 
    "SearchResult",
    "FinderConfig",
    
    # 核心类
    "PriorityDirectoryFinder",
    "QuickFinder",
    
    # 便捷函数
    "find_command_directories",
    "find_config_directories",
    "find_standard_directories",
    "create_file_filter",
    "create_directory_config",
]


def find_command_directories(current_dir=None):
    """
    查找命令目录的便捷函数
    
    Args:
        current_dir: 当前目录，如果为None则使用os.getcwd()
        
    Returns:
        SearchResult: 查找结果
    """
    return QuickFinder.find_command_directories(current_dir)


def find_config_directories(current_dir=None):
    """
    查找配置目录的便捷函数
    
    Args:
        current_dir: 当前目录
        
    Returns:
        SearchResult: 查找结果
    """
    return QuickFinder.find_config_directories(current_dir)


def find_standard_directories(base_name, current_dir=None, file_extensions=None, strategy=SearchStrategy.FIRST_MATCH):
    """
    查找标准目录结构的便捷函数
    
    Args:
        base_name: 基础目录名
        current_dir: 当前目录
        file_extensions: 要查找的文件扩展名列表
        strategy: 查找策略
        
    Returns:
        SearchResult: 查找结果
    """
    return QuickFinder.find_standard_directories(base_name, current_dir, file_extensions, strategy)


def create_file_filter(extensions=None, patterns=None, min_count=1, recursive=True, include_hidden=False):
    """
    创建文件类型过滤器的便捷函数
    
    Args:
        extensions: 文件扩展名列表
        patterns: 文件名模式列表
        min_count: 最小文件数量
        recursive: 是否递归搜索
        include_hidden: 是否包含隐藏文件
        
    Returns:
        FileTypeFilter: 文件过滤器
    """
    return FileTypeFilter(
        extensions=set(extensions or []),
        patterns=set(patterns or []),
        min_count=min_count,
        recursive=recursive,
        include_hidden=include_hidden
    )


def create_directory_config(path, priority=0, validation_mode=ValidationMode.HAS_FILES, 
                          file_filter=None, description=""):
    """
    创建目录配置的便捷函数
    
    Args:
        path: 目录路径
        priority: 优先级
        validation_mode: 验证模式
        file_filter: 文件过滤器
        description: 描述
        
    Returns:
        DirectoryConfig: 目录配置
    """
    return DirectoryConfig(
        path=path,
        priority=priority,
        validation_mode=validation_mode,
        file_filter=file_filter,
        description=description
    ) 


"""
Rules command messages for internationalization
Contains all messages used by the rules command functionality
"""

RULES_COMMAND_MESSAGES = {
    "rule_cmd_analyzing_project": {
        "en": "Using subagent mechanism to analyze project structure and generate initialization rules...",
        "zh": "正在使用 subagent 机制分析项目结构并生成初始化规则...",
        "ja": "サブエージェント機構を使用してプロジェクト構造を分析し、初期化ルールを生成しています...",
        "ar": "استخدام آلية الوكيل الفرعي لتحليل هيكل المشروع وإنشاء قواعد التهيئة...",
        "ru": "Использование механизма подагента для анализа структуры проекта и генерации правил инициализации..."
    },
    "rule_cmd_init_failed": {
        "en": "Initialization rules generation failed, please check project directory and model configuration",
        "zh": "初始化规则生成失败，请检查项目目录和模型配置",
        "ja": "初期化ルールの生成に失敗しました。プロジェクトディレクトリとモデル設定を確認してください",
        "ar": "فشل في إنشاء قواعد التهيئة، يرجى التحقق من دليل المشروع وتكوين النموذج",
        "ru": "Не удалось сгенерировать правила инициализации, проверьте директорию проекта и конфигурацию модели"
    },
    "rule_cmd_init_success": {
        "en": "Initialization rules generated successfully!",
        "zh": "初始化规则生成成功！",
        "ja": "初期化ルールが正常に生成されました！",
        "ar": "تم إنشاء قواعد التهيئة بنجاح！",
        "ru": "Правила инициализации успешно сгенерированы!"
    },
    "rule_cmd_project_type": {
        "en": "Project type",
        "zh": "项目类型",
        "ja": "プロジェクトタイプ",
        "ar": "نوع المشروع",
        "ru": "Тип проекта"
    },
    "rule_cmd_rules_file": {
        "en": "Rules file",
        "zh": "规则文件",
        "ja": "ルールファイル",
        "ar": "ملف القواعد",
        "ru": "Файл правил"
    },
    "rule_cmd_detected_technologies": {
        "en": "Detected technologies",
        "zh": "检测到的技术栈",
        "ja": "検出された技術",
        "ar": "التقنيات المكتشفة",
        "ru": "Обнаруженные технологии"
    },
    "rule_cmd_detected_commands": {
        "en": "Detected commands",
        "zh": "检测到的命令",
        "ja": "検出されたコマンド",
        "ar": "الأوامر المكتشفة",
        "ru": "Обнаруженные команды"
    },
    "rule_cmd_rules_content_preview": {
        "en": "Rules content preview",
        "zh": "规则内容预览",
        "ja": "ルール内容プレビュー",
        "ar": "معاينة محتوى القواعد",
        "ru": "Предпросмотр содержимого правил"
    },
    "rule_cmd_edit_rules_note": {
        "en": "You can edit the generated rules file to customize project development standards.",
        "zh": "您可以编辑生成的规则文件来自定义项目开发规范。",
        "ja": "生成されたルールファイルを編集して、プロジェクト開発標準をカスタマイズできます。",
        "ar": "يمكنك تحرير ملف القواعد المُنشأ لتخصيص معايير تطوير المشروع.",
        "ru": "Вы можете отредактировать сгенерированный файл правил для настройки стандартов разработки проекта."
    },
    "rule_cmd_init_error": {
        "en": "Error occurred during initialization rules generation: {{error}}",
        "zh": "初始化规则生成时出错: {{error}}",
        "ja": "初期化ルール生成中にエラーが発生しました: {{error}}",
        "ar": "حدث خطأ أثناء إنشاء قواعد التهيئة: {{error}}",
        "ru": "Произошла ошибка при генерации правил инициализации: {{error}}"
    }
}


"""
Similarity-based text replacer implementation
"""

from typing import List, Tuple
from loguru import logger

from .base import BaseReplacer, ReplaceResult, ReplaceStrategy
from ..text_similarity import TextSimilarity


class SimilarityReplacer(BaseReplacer):
    """基于文本相似度的替换器"""
    
    def __init__(self, similarity_threshold: float = 0.9):
        """
        初始化相似度替换器
        
        Args:
            similarity_threshold: 相似度阈值，超过此值才进行替换
        """
        super().__init__(ReplaceStrategy.SIMILARITY)
        self.similarity_threshold = similarity_threshold
    
    def _find_line_numbers(self, content: str, text_block: str) -> Tuple[int, int]:
        """
        找到文本块在内容中的行号
        
        Args:
            content: 完整文件内容
            text_block: 要查找的文本块
            
        Returns:
            Tuple[int, int]: (起始行号, 结束行号)，1-based，如果未找到返回 (-1, -1)
        """
        lines = content.splitlines()
        block_lines = text_block.splitlines()
        
        # 找到文本块在内容中的位置
        block_start_idx = content.find(text_block)
        if block_start_idx == -1:
            return (-1, -1)
        
        # 计算到文本块开始位置的行数
        lines_before = content[:block_start_idx].count('\n')
        start_line = lines_before + 1
        
        # 计算文本块的行数
        lines_in_block = len(block_lines)
        end_line = start_line + lines_in_block - 1
        
        return (start_line, end_line)
    
    def _replace_by_line_numbers(self, content: str, start_line: int, end_line: int, 
                                 replace_text: str) -> str:
        """
        根据行号替换文本
        
        Args:
            content: 原始内容
            start_line: 起始行号（1-based）
            end_line: 结束行号（1-based）
            replace_text: 替换的文本
            
        Returns:
            替换后的内容
        """
        lines = content.splitlines(keepends=True)
        
        # 转换为0-based索引
        start_idx = start_line - 1
        end_idx = end_line - 1
        
        # 确保索引有效
        if start_idx < 0 or end_idx >= len(lines) or start_idx > end_idx:
            return content
        
        # 构建新内容
        new_lines = []
        new_lines.extend(lines[:start_idx])
        
        # 添加替换文本
        replace_lines = replace_text.splitlines(keepends=True)
        # 如果替换文本最后没有换行符，但原始文本有，则添加换行符
        if replace_lines and not replace_lines[-1].endswith('\n') and end_idx < len(lines) - 1:
            replace_lines[-1] += '\n'
        new_lines.extend(replace_lines)
        
        new_lines.extend(lines[end_idx + 1:])
        
        return ''.join(new_lines)
    
    def _build_similarity_error_message(self, block_index: int, search_text: str, similarity: float, 
                                       threshold: float, best_match: str, start_line: int, end_line: int) -> str:
        """
        构建针对大模型的详细相似度错误信息
        """
        message_parts = []
        
        message_parts.append(f"🔍 Block {block_index} Similarity Analysis:")
        message_parts.append(f"   📏 Found similarity: {similarity:.1%} (threshold: {threshold:.1%})")
        
        if similarity > 0.8:
            message_parts.append(f"   ✅ Very close match found at lines {start_line}-{end_line}")
            message_parts.append(f"   📝 Expected: {repr(search_text[:80])}{'...' if len(search_text) > 80 else ''}")
            message_parts.append(f"   📝 Actual:   {repr(best_match[:80])}{'...' if len(best_match) > 80 else ''}")
            message_parts.append("   💡 Use the exact content shown as 'Actual' in your SEARCH block")
        elif similarity > 0.6:
            message_parts.append(f"   ⚠️  Moderate similarity at lines {start_line}-{end_line}")
            message_parts.append(f"   📝 Your search: {repr(search_text[:60])}{'...' if len(search_text) > 60 else ''}")
            message_parts.append(f"   📝 Best match:  {repr(best_match[:60])}{'...' if len(best_match) > 60 else ''}")
            message_parts.append("   💡 Check for whitespace, line ending, or formatting differences")
        else:
            message_parts.append(f"   ❌ Low similarity - content may not exist as expected")
            message_parts.append(f"   📝 Searched for: {repr(search_text[:60])}{'...' if len(search_text) > 60 else ''}")
            message_parts.append("   💡 Verify the content exists or try a smaller, more unique search phrase")
        
        return "\n".join(message_parts)
    
    def replace(self, content: str, search_blocks: List[Tuple[str, str]]) -> ReplaceResult:
        """
        执行基于相似度的替换（行级替换）
        
        Args:
            content: 原始文件内容
            search_blocks: 搜索替换块列表
            
        Returns:
            ReplaceResult: 替换结果
        """
        if not self.validate_search_blocks(search_blocks):
            return ReplaceResult(
                success=False,
                message="Invalid search blocks provided. 只支持行级替换。",
                total_count=len(search_blocks)
            )
        
        current_content = content
        applied_count = 0
        errors = []
        similarity_info = []
        
        for i, (search_text, replace_text) in enumerate(search_blocks):
            try:
                # ----- 插入操作 -----
                if search_text == "":
                    # 插入到文件开头
                    insert_text = replace_text
                    # 如果插入文本不以换行符结尾，自动补一个，保持行级语义
                    if not insert_text.endswith(("\n", "\r\n")):
                        insert_text += "\n"
                    
                    current_content = insert_text + current_content
                    applied_count += 1
                    logger.info(f"Similarity replacer inserted block {i+1}/{len(search_blocks)} at file head")
                    continue
                
                # ----- 正常替换操作 -----
                # 硬约束：检查是否为行级替换
                if not self._is_line_level_search(search_text, content):
                    error_msg = f"🔍 Block {i+1}: 只支持行级替换\n   📝 Search text: {repr(search_text[:60])}{'...' if len(search_text) > 60 else ''}\n   💡 请确保搜索文本匹配完整行"
                    errors.append(error_msg)
                    continue
                
                # 使用TextSimilarity查找最佳匹配
                similarity_finder = TextSimilarity(search_text, current_content)
                similarity, best_window = similarity_finder.get_best_matching_window()
                
                logger.info(f"Block {i+1}: Found similarity {similarity:.3f} with threshold {self.similarity_threshold}")
                
                if similarity >= self.similarity_threshold:
                    # 找到行号
                    start_line, end_line = self._find_line_numbers(current_content, best_window)
                    
                    if start_line > 0 and end_line > 0:
                        # 执行替换
                        new_content = self._replace_by_line_numbers(
                            current_content, start_line, end_line, replace_text
                        )
                        
                        if new_content != current_content:
                            current_content = new_content
                            applied_count += 1
                            similarity_info.append({
                                'block_index': i + 1,
                                'similarity': similarity,
                                'start_line': start_line,
                                'end_line': end_line,
                                'matched_text': best_window[:100] + '...' if len(best_window) > 100 else best_window
                            })
                            logger.info(f"Applied block {i+1} at lines {start_line}-{end_line}")
                        else:
                            error_msg = f"Block {i+1}: Replacement did not change content"
                            errors.append(error_msg)
                            logger.warning(error_msg)
                    else:
                        error_msg = f"Block {i+1}: Could not determine line numbers for matched text"
                        errors.append(error_msg)
                        logger.warning(error_msg)
                else:
                    # 构建详细的错误信息给大模型
                    start_line, end_line = self._find_line_numbers(current_content, best_window)
                    error_msg = self._build_similarity_error_message(
                        i + 1, search_text, similarity, self.similarity_threshold, 
                        best_window, start_line, end_line
                    )
                    errors.append(error_msg)
                    logger.warning(f"Block {i+1}: Similarity {similarity:.3f} below threshold {self.similarity_threshold:.3f}")
                    
                    # 提供调试信息
                    similarity_info.append({
                        'block_index': i + 1,
                        'similarity': similarity,
                        'threshold': self.similarity_threshold,
                        'best_match': best_window[:100] + '...' if len(best_window) > 100 else best_window,
                        'location': f"lines {start_line}-{end_line}" if start_line > 0 else "unknown"
                    })
                    
            except Exception as e:
                error_msg = f"Block {i+1}: Error during similarity matching - {str(e)}"
                errors.append(error_msg)
                logger.error(error_msg)
        
        success = applied_count > 0
        
        if success:
            if errors:
                message = f"Partial success: Applied {applied_count}/{len(search_blocks)} blocks with {len(errors)} errors"
            else:
                message = f"Success: Applied {applied_count}/{len(search_blocks)} blocks using similarity matching"
        else:
            message = f"Failed: No blocks were applied using similarity matching. {len(errors)} errors occurred"
        
        return ReplaceResult(
            success=success,
            message=message,
            new_content=current_content if success else None,
            applied_count=applied_count,
            total_count=len(search_blocks),
            errors=errors,
            metadata={
                'strategy': self.get_strategy_name(),
                'similarity_threshold': self.similarity_threshold,
                'similarity_info': similarity_info
            }
        )
    
    def can_handle(self, content: str, search_blocks: List[Tuple[str, str]]) -> bool:
        """
        检查是否能处理给定的内容和搜索块
        
        Args:
            content: 原始文件内容
            search_blocks: 搜索替换块列表
            
        Returns:
            bool: 总是返回 True，因为相似度替换器可以处理所有文本
        """
        return self.validate_search_blocks(search_blocks)
    
    def set_similarity_threshold(self, threshold: float):
        """
        设置相似度阈值
        
        Args:
            threshold: 新的相似度阈值 (0.0 - 1.0)
        """
        if 0.0 <= threshold <= 1.0:
            self.similarity_threshold = threshold
            logger.info(f"Similarity threshold set to {threshold}")
        else:
            logger.warning(f"Invalid similarity threshold {threshold}, must be between 0.0 and 1.0")
    
    def _is_line_level_search(self, search_text: str, content: str) -> bool:
        """
        检查搜索文本是否为行级搜索
        
        Args:
            search_text: 搜索文本
            content: 文件内容
            
        Returns:
            bool: 是否为行级搜索
        """
        # 如果搜索文本包含换行符，认为是多行搜索，允许
        if '\n' in search_text:
            return True
        
        # 对于单行文本，使用TextSimilarity检查是否能找到完整行匹配
        try:
            similarity_finder = TextSimilarity(search_text, content)
            similarity, best_window = similarity_finder.get_best_matching_window()
            
            # 检查最佳匹配是否为完整行
            if similarity >= self.similarity_threshold:
                content_lines = content.splitlines()
                best_window_stripped = best_window.strip()
                
                # 检查最佳匹配是否等于某一完整行
                for line in content_lines:
                    if line.strip() == best_window_stripped:
                        return True
            
            return False
        except Exception:
            return False 


"""布局定义"""

from prompt_toolkit.layout.containers import (
    HSplit,
    VSplit,
    Window,
    WindowAlign,
    ConditionalContainer,
)
from prompt_toolkit.layout.controls import FormattedTextControl, BufferControl
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.layout.dimension import Dimension as D
from prompt_toolkit.filters import Condition
from prompt_toolkit.widgets import Frame


def create_layout(
    conversation_buffer,
    input_buffer,
    get_prompt_text,
    get_right_hint,
    get_tips_text,
    get_shortcuts_help_text,
    show_shortcuts_condition,
    input_has_text_condition,
):
    """创建应用布局"""

    # 创建对话历史控件
    conversation_control = FormattedTextControl(
        text=conversation_buffer.get_formatted_text,
        show_cursor=False,
        focusable=False,
    )

    # 创建输入控件
    input_control = BufferControl(
        buffer=input_buffer, search_buffer_control=None, focus_on_click=True
    )

    # 创建欢迎面板
    welcome_content = Window(
        content=FormattedTextControl(
            text=conversation_buffer.get_welcome_text, show_cursor=False
        ),
        style="class:welcome-panel",
    )

    welcome_panel = Frame(body=welcome_content, title="", style="class:welcome-border")

    # 对话历史窗口
    conversation_window = Window(
        content=conversation_control,
        wrap_lines=True,
        style="class:conversation",
        always_hide_cursor=True,
        height=D(min=5),
        dont_extend_height=False,
        scroll_offsets=None,
    )

    # 主要内容区域
    main_content = VSplit(
        [
            # 左侧留白
            Window(width=D(min=2, max=2)),
            # 中间内容
            HSplit(
                [
                    # 欢迎面板
                    welcome_panel,
                    # Tips 部分
                    Window(
                        content=FormattedTextControl(
                            text=get_tips_text, show_cursor=False
                        ),
                        height=D(min=7, max=7),
                        style="class:tips",
                    ),
                    # 对话历史区域
                    conversation_window,
                ]
            ),
            # 右侧留白
            Window(width=D(min=2, max=2)),
        ]
    )

    # 输入区域
    input_area = VSplit(
        [
            # 输入提示符
            Window(
                content=FormattedTextControl(text=get_prompt_text),
                width=D(min=2, max=4),
                style="class:prompt",
            ),
            # 输入框
            Window(content=input_control, style="class:input", height=D(min=1, max=1)),
            # 右侧提示信息
            Window(
                content=FormattedTextControl(text=get_right_hint),
                width=D(min=30, max=30),
                style="class:hint-right",
                align=WindowAlign.RIGHT,
            ),
        ],
        height=D(min=1, max=1),
    )

    # 创建布局
    root_container = HSplit(
        [
            # 主内容区域
            HSplit([main_content]),
            # 底部分隔
            Window(height=D(min=1, max=1)),
            # 输入区域边框
            Window(height=D(min=1, max=1), char="─", style="class:input-separator"),
            # 输入区域
            input_area,
            # 底部快捷键帮助
            ConditionalContainer(
                content=Window(
                    content=FormattedTextControl(
                        text=get_shortcuts_help_text, show_cursor=False
                    ),
                    height=D(min=5, max=5),
                    style="class:shortcuts-help",
                ),
                filter=Condition(
                    lambda: show_shortcuts_condition()
                    and not input_has_text_condition()
                ),
            ),
            # 底部空白
            Window(height=D(min=1, max=1)),
        ]
    )

    return Layout(root_container, focused_element=input_buffer)



from autocoder.common import (
    AutoCoderArgs,
    TranslateArgs,
    TranslateReadme,
    split_code_into_segments,
    SourceCode,
)
from autocoder.common.buildin_tokenizer import BuildinTokenizer
from autocoder.pyproject import PyProject
from autocoder.tsproject import TSProject
from autocoder.default_project import DefaultProject
from autocoder.suffixproject import SuffixProject
from autocoder.index.entry import build_index_and_filter_files
from autocoder.common.code_auto_merge import CodeAutoMerge
from autocoder.common.code_auto_merge_diff import CodeAutoMergeDiff
from autocoder.common.code_auto_merge_strict_diff import CodeAutoMergeStrictDiff
from autocoder.common.code_auto_merge_editblock import CodeAutoMergeEditBlock
from autocoder.common.code_auto_generate import CodeAutoGenerate
from autocoder.common.code_auto_generate_diff import CodeAutoGenerateDiff
from autocoder.common.code_auto_generate_strict_diff import CodeAutoGenerateStrictDiff
from autocoder.common.code_auto_generate_editblock import CodeAutoGenerateEditBlock
from typing import Optional, Generator
import byzerllm
import os
from autocoder.common.image_to_page import ImageToPage, ImageToPageDirectly
from autocoder.utils.conversation_store import store_code_model_conversation
from loguru import logger
import time
from autocoder.common.printer import Printer
from autocoder.utils.llms import get_llm_names
from autocoder.privacy.model_filter import ModelPathFilter
from autocoder.common import SourceCodeList
from autocoder.common.global_cancel import global_cancel
from autocoder.events.event_manager_singleton import get_event_manager
from autocoder.events import event_content as EventContentCreator
from autocoder.events.event_types import EventMetadata
from autocoder.common.v2.code_editblock_manager import CodeEditBlockManager
from autocoder.common.conversations.get_conversation_manager import (
    get_conversation_manager,
)


class BaseAction:
    def _get_content_length(self, content: str) -> int:
        try:
            tokenizer = BuildinTokenizer()
            return tokenizer.count_tokens(content)
        except Exception as e:
            logger.warning(
                f"Failed to use tokenizer to count tokens, fallback to len(): {e}"
            )
            return len(content)

    def _add_conversations_to_manager(
        self, result, conversation_manager, namespace=None
    ):
        """
        将生成结果中的对话添加到 conversation_manager

        参数:
            result: CodeGenerateResult 对象
            conversation_manager: ConversationManager 实例
            namespace: 命名空间（可选）
        """
        if not result.conversations:
            return

        # 确保有当前会话，如果没有则创建
        current_conversation_id = conversation_manager.get_current_conversation_id(
            namespace
        )
        if not current_conversation_id:
            current_conversation_id = conversation_manager.create_conversation(
                name="Code Generation Session",
                description="Auto-coder code generation session",
            )
            conversation_manager.set_current_conversation(
                current_conversation_id, namespace
            )

        # 添加对话到当前会话
        # result.conversations 是 List[List[Dict[str, Any]]]，取最后一轮对话
        if result.conversations:
            last_conversation = result.conversations[-1]
            for message in last_conversation:
                if (
                    isinstance(message, dict)
                    and "role" in message
                    and "content" in message
                ):
                    conversation_manager.append_message_to_current(
                        role=message["role"],
                        content=message["content"],
                        namespace=namespace,
                    )


class ActionTSProject(BaseAction):
    def __init__(
        self, args: AutoCoderArgs, llm: Optional[byzerllm.ByzerLLM] = None
    ) -> None:
        self.args = args
        self.llm = llm
        self.pp = None
        self.printer = Printer()
        self.conversation_manager = get_conversation_manager()
        self.namespace = None

    def run(self):
        args = self.args
        if args.project_type != "ts":
            return False
        pp = TSProject(args=args, llm=self.llm)
        self.pp = pp
        pp.run()

        # source_code = pp.output()
        source_code_list = SourceCodeList(pp.sources)
        if self.llm:
            if args.in_code_apply:
                old_query = args.query
                args.query = (args.context or "") + "\n\n" + args.query
            source_code_list = build_index_and_filter_files(
                llm=self.llm, args=args, sources=pp.sources
            )
            if args.in_code_apply:
                args.query = old_query

        if args.image_file:
            if args.image_mode == "iterative":
                image_to_page = ImageToPage(llm=self.llm, args=args)
            else:
                image_to_page = ImageToPageDirectly(llm=self.llm, args=args)

            file_name = os.path.splitext(os.path.basename(args.image_file))[0]
            html_path = os.path.join(
                os.path.dirname(args.image_file), "html", f"{file_name}.html"
            )
            image_to_page.run_then_iterate(
                origin_image=args.image_file,
                html_path=html_path,
                max_iter=self.args.image_max_iter,
            )
            html_code = ""
            with open(html_path, "r", encoding="utf-8") as f:
                html_code = f.read()

            source_code_list.sources.append(
                SourceCode(module_name=html_path, source_code=html_code, tag="IMAGE")
            )

        self.process_content(source_code_list)
        return True

    def process_content(self, source_code_list: SourceCodeList):
        args = self.args
        content = source_code_list.to_str()
        if args.execute and self.llm and not args.human_as_model:
            content_length = self._get_content_length(content)
            if content_length > self.args.model_max_input_length:
                logger.warning(
                    f"Content(send to model) is {content_length} tokens, which is larger than the maximum input length {self.args.model_max_input_length}"
                )

        global_cancel.check_and_raise(token=self.args.cancel_token)

        if (
            (args.enable_auto_fix_merge or args.enable_auto_fix_lint)
            and args.execute
            and args.auto_merge == "editblock"
        ):
            code_merge_manager = CodeEditBlockManager(
                llm=self.llm, args=self.args, action=self
            )
            result = code_merge_manager.run(
                query=args.query, source_code_list=source_code_list
            )
            # 将对话添加到 conversation_manager
            self._add_conversations_to_manager(
                result, self.conversation_manager, self.namespace
            )
            return

        if args.execute:
            self.printer.print_in_terminal("code_generation_start")
            start_time = time.time()
            if args.auto_merge == "diff":
                generate = CodeAutoGenerateDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "strict_diff":
                generate = CodeAutoGenerateStrictDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "editblock":
                generate = CodeAutoGenerateEditBlock(
                    llm=self.llm, args=self.args, action=self
                )
            else:
                generate = CodeAutoGenerate(llm=self.llm, args=self.args, action=self)

            generate_result = generate.single_round_run(
                query=args.query, source_code_list=source_code_list
            )
            elapsed_time = time.time() - start_time
            speed = (
                generate_result.metadata.get("generated_tokens_count", 0) / elapsed_time
                if elapsed_time > 0
                else 0
            )
            input_tokens_cost = generate_result.metadata.get("input_tokens_cost", 0)
            generated_tokens_cost = generate_result.metadata.get(
                "generated_tokens_cost", 0
            )
            model_names = ",".join(get_llm_names(generate.llms))
            self.printer.print_in_terminal(
                "code_generation_complete",
                duration=elapsed_time,
                input_tokens=generate_result.metadata.get("input_tokens_count", 0),
                output_tokens=generate_result.metadata.get("generated_tokens_count", 0),
                input_cost=input_tokens_cost,
                output_cost=generated_tokens_cost,
                speed=round(speed, 2),
                model_names=model_names,
                sampling_count=len(generate_result.contents),
            )

            get_event_manager(self.args.event_file).write_result(
                EventContentCreator.create_result(
                    content=EventContentCreator.ResultTokenStatContent(
                        model_name=model_names,
                        elapsed_time=elapsed_time,
                        input_tokens=generate_result.metadata.get(
                            "input_tokens_count", 0
                        ),
                        output_tokens=generate_result.metadata.get(
                            "generated_tokens_count", 0
                        ),
                        input_cost=input_tokens_cost,
                        output_cost=generated_tokens_cost,
                        speed=round(speed, 2),
                    )
                ).to_dict(),
                metadata=EventMetadata(action_file=self.args.file).to_dict(),
            )

            global_cancel.check_and_raise(token=self.args.cancel_token)

            merge_result = None
            if args.execute and args.auto_merge:
                self.printer.print_in_terminal("code_merge_start")
                if args.auto_merge == "diff":
                    code_merge = CodeAutoMergeDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "strict_diff":
                    code_merge = CodeAutoMergeStrictDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "editblock":
                    code_merge = CodeAutoMergeEditBlock(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                else:
                    code_merge = CodeAutoMerge(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )

                if merge_result is not None:
                    content = merge_result.contents[0]
                    store_code_model_conversation(
                        args=self.args,
                        instruction=self.args.query,
                        conversations=merge_result.conversations[0],
                        model=self.llm.default_model_name,
                    )
                else:
                    content = generate_result.contents[0]
                    store_code_model_conversation(
                        args=self.args,
                        instruction=self.args.query,
                        conversations=generate_result.conversations[0],
                        model=self.llm.default_model_name,
                    )


class ActionPyProject(BaseAction):
    def __init__(
        self, args: AutoCoderArgs, llm: Optional[byzerllm.ByzerLLM] = None
    ) -> None:
        self.args = args
        self.llm = llm
        self.pp = None
        self.printer = Printer()
        self.conversation_manager = get_conversation_manager()
        self.namespace = None

    def run(self):
        args = self.args
        if args.project_type != "py":
            return False
        pp = PyProject(args=self.args, llm=self.llm)
        self.pp = pp
        pp.run(packages=args.py_packages.split(",") if args.py_packages else [])
        source_code_list = SourceCodeList(pp.sources)

        if self.llm:
            old_query = args.query
            if args.in_code_apply:
                args.query = (args.context or "") + "\n\n" + args.query
            source_code_list = build_index_and_filter_files(
                llm=self.llm, args=args, sources=pp.sources
            )
            if args.in_code_apply:
                args.query = old_query

        self.process_content(source_code_list)
        return True

    def process_content(self, source_code_list: SourceCodeList):
        args = self.args
        content = source_code_list.to_str()
        if args.execute and self.llm and not args.human_as_model:
            content_length = self._get_content_length(content)
            if content_length > self.args.model_max_input_length:
                self.printer.print_in_terminal(
                    "code_execution_warning",
                    style="yellow",
                    content_length=content_length,
                    max_length=self.args.model_max_input_length,
                )

        global_cancel.check_and_raise(token=self.args.cancel_token)

        if (
            (args.enable_auto_fix_merge or args.enable_auto_fix_lint)
            and args.execute
            and args.auto_merge == "editblock"
        ):
            code_merge_manager = CodeEditBlockManager(
                llm=self.llm, args=self.args, action=self
            )
            result = code_merge_manager.run(
                query=args.query, source_code_list=source_code_list
            )
            # 将对话添加到 conversation_manager
            self._add_conversations_to_manager(
                result, self.conversation_manager, self.namespace
            )
            return

        if args.execute:
            self.printer.print_in_terminal("code_generation_start")
            start_time = time.time()
            if args.auto_merge == "diff":
                generate = CodeAutoGenerateDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "strict_diff":
                generate = CodeAutoGenerateStrictDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "editblock":
                generate = CodeAutoGenerateEditBlock(
                    llm=self.llm, args=self.args, action=self
                )
            else:
                generate = CodeAutoGenerate(llm=self.llm, args=self.args, action=self)

            generate_result = generate.single_round_run(
                query=args.query, source_code_list=source_code_list
            )

            elapsed_time = time.time() - start_time
            speed = (
                generate_result.metadata.get("generated_tokens_count", 0) / elapsed_time
                if elapsed_time > 0
                else 0
            )
            model_names = ",".join(get_llm_names(generate.llms))
            input_tokens_cost = generate_result.metadata.get("input_tokens_cost", 0)
            generated_tokens_cost = generate_result.metadata.get(
                "generated_tokens_cost", 0
            )
            self.printer.print_in_terminal(
                "code_generation_complete",
                duration=elapsed_time,
                input_tokens=generate_result.metadata.get("input_tokens_count", 0),
                output_tokens=generate_result.metadata.get("generated_tokens_count", 0),
                input_cost=input_tokens_cost,
                output_cost=generated_tokens_cost,
                speed=round(speed, 2),
                model_names=model_names,
                sampling_count=len(generate_result.contents),
            )

            get_event_manager(self.args.event_file).write_result(
                EventContentCreator.create_result(
                    content=EventContentCreator.ResultTokenStatContent(
                        model_name=model_names,
                        elapsed_time=elapsed_time,
                        input_tokens=generate_result.metadata.get(
                            "input_tokens_count", 0
                        ),
                        output_tokens=generate_result.metadata.get(
                            "generated_tokens_count", 0
                        ),
                        input_cost=input_tokens_cost,
                        output_cost=generated_tokens_cost,
                        speed=round(speed, 2),
                    )
                ).to_dict(),
                metadata=EventMetadata(action_file=self.args.file).to_dict(),
            )

            global_cancel.check_and_raise(token=self.args.cancel_token)

            merge_result = None
            if args.execute and args.auto_merge:
                self.printer.print_in_terminal("code_merge_start")
                if args.auto_merge == "diff":
                    code_merge = CodeAutoMergeDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "strict_diff":
                    code_merge = CodeAutoMergeStrictDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "editblock":
                    code_merge = CodeAutoMergeEditBlock(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                else:
                    code_merge = CodeAutoMerge(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )

                content = merge_result.contents[0]

                store_code_model_conversation(
                    args=self.args,
                    instruction=self.args.query,
                    conversations=merge_result.conversations[0],
                    model=self.llm.default_model_name,
                )
            else:
                content = generate_result.contents[0]

                store_code_model_conversation(
                    args=self.args,
                    instruction=self.args.query,
                    conversations=generate_result.conversations[0],
                    model=self.llm.default_model_name,
                )


class ActionSuffixProject(BaseAction):
    def __init__(
        self, args: AutoCoderArgs, llm: Optional[byzerllm.ByzerLLM] = None
    ) -> None:
        self.args = args
        self.llm = llm
        self.pp = None
        self.printer = Printer()
        self.conversation_manager = get_conversation_manager()
        self.namespace = None

    def run(self):
        args = self.args
        pp = SuffixProject(args=args, llm=self.llm)
        self.pp = pp
        pp.run()
        source_code_list = SourceCodeList(pp.sources)
        if self.llm:
            if args.in_code_apply:
                old_query = args.query
                args.query = (args.context or "") + "\n\n" + args.query
            source_code_list = build_index_and_filter_files(
                llm=self.llm, args=args, sources=pp.sources
            )
            if args.in_code_apply:
                args.query = old_query
        self.process_content(source_code_list)

    def process_content(self, source_code_list: SourceCodeList):
        args = self.args
        content = source_code_list.to_str()

        if args.execute and self.llm and not args.human_as_model:
            content_length = self._get_content_length(content)
            if content_length > self.args.model_max_input_length:
                logger.warning(
                    f"Content(send to model) is {content_length} tokens, which is larger than the maximum input length {self.args.model_max_input_length}"
                )

        global_cancel.check_and_raise(token=self.args.cancel_token)

        if (
            (args.enable_auto_fix_merge or args.enable_auto_fix_lint)
            and args.execute
            and args.auto_merge == "editblock"
        ):
            code_merge_manager = CodeEditBlockManager(
                llm=self.llm, args=self.args, action=self
            )
            result = code_merge_manager.run(
                query=args.query, source_code_list=source_code_list
            )
            # 将对话添加到 conversation_manager
            self._add_conversations_to_manager(
                result, self.conversation_manager, self.namespace
            )
            return

        if args.execute:
            self.printer.print_in_terminal("code_generation_start")
            start_time = time.time()
            if args.auto_merge == "diff":
                generate = CodeAutoGenerateDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "strict_diff":
                generate = CodeAutoGenerateStrictDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "editblock":
                generate = CodeAutoGenerateEditBlock(
                    llm=self.llm, args=self.args, action=self
                )
            else:
                generate = CodeAutoGenerate(llm=self.llm, args=self.args, action=self)

            generate_result = generate.single_round_run(
                query=args.query, source_code_list=source_code_list
            )

        elapsed_time = time.time() - start_time
        speed = (
            generate_result.metadata.get("generated_tokens_count", 0) / elapsed_time
            if elapsed_time > 0
            else 0
        )
        model_names = ",".join(get_llm_names(generate.llms))
        input_tokens_cost = generate_result.metadata.get("input_tokens_cost", 0)
        generated_tokens_cost = generate_result.metadata.get("generated_tokens_cost", 0)
        self.printer.print_in_terminal(
            "code_generation_complete",
            duration=elapsed_time,
            input_tokens=generate_result.metadata.get("input_tokens_count", 0),
            output_tokens=generate_result.metadata.get("generated_tokens_count", 0),
            input_cost=input_tokens_cost,
            output_cost=generated_tokens_cost,
            speed=round(speed, 2),
            model_names=model_names,
            sampling_count=len(generate_result.contents),
        )

        get_event_manager(self.args.event_file).write_result(
            EventContentCreator.create_result(
                content=EventContentCreator.ResultTokenStatContent(
                    model_name=model_names,
                    elapsed_time=elapsed_time,
                    input_tokens=generate_result.metadata.get("input_tokens_count", 0),
                    output_tokens=generate_result.metadata.get(
                        "generated_tokens_count", 0
                    ),
                    input_cost=input_tokens_cost,
                    output_cost=generated_tokens_cost,
                    speed=round(speed, 2),
                )
            ).to_dict(),
            metadata=EventMetadata(action_file=self.args.file).to_dict(),
        )

        global_cancel.check_and_raise(token=self.args.cancel_token)

        merge_result = None
        if args.execute and args.auto_merge:
            self.printer.print_in_terminal("code_merge_start")
            if args.auto_merge == "diff":
                code_merge = CodeAutoMergeDiff(llm=self.llm, args=self.args)
                merge_result = code_merge.merge_code(generate_result=generate_result)
            elif args.auto_merge == "strict_diff":
                code_merge = CodeAutoMergeStrictDiff(llm=self.llm, args=self.args)
                merge_result = code_merge.merge_code(generate_result=generate_result)
            elif args.auto_merge == "editblock":
                code_merge = CodeAutoMergeEditBlock(llm=self.llm, args=self.args)
                merge_result = code_merge.merge_code(generate_result=generate_result)
            else:
                code_merge = CodeAutoMerge(llm=self.llm, args=self.args)
                merge_result = code_merge.merge_code(generate_result=generate_result)

        if merge_result is not None:
            content = merge_result.contents[0]
            store_code_model_conversation(
                args=self.args,
                instruction=self.args.query,
                conversations=merge_result.conversations[0],
                model=self.llm.default_model_name,
            )
        else:
            content = generate_result.contents[0]

            store_code_model_conversation(
                args=self.args,
                instruction=self.args.query,
                conversations=generate_result.conversations[0],
                model=self.llm.default_model_name,
            )


class ActionDefaultProject(BaseAction):
    """
    ActionDefaultProject用于处理默认项目类型，使用ignorefiles模块进行文件过滤。
    与ActionSuffixProject不同，它不按文件后缀过滤，而是收集所有不被忽略的文本文件。
    """

    def __init__(
        self, args: AutoCoderArgs, llm: Optional[byzerllm.ByzerLLM] = None
    ) -> None:
        self.args = args
        self.llm = llm
        self.pp = None
        self.printer = Printer()
        self.conversation_manager = get_conversation_manager()
        self.namespace = None

    def run(self):
        args = self.args

        # project_type为*时，或者没有设置project_type时，执行默认项目
        if args.project_type and args.project_type != "*":
            return False

        pp = DefaultProject(args=args, llm=self.llm)
        self.pp = pp
        pp.run()
        source_code_list = SourceCodeList(pp.sources)
        if self.llm:
            if args.in_code_apply:
                old_query = args.query
                args.query = (args.context or "") + "\n\n" + args.query
            source_code_list = build_index_and_filter_files(
                llm=self.llm, args=args, sources=pp.sources
            )
            if args.in_code_apply:
                args.query = old_query
        self.process_content(source_code_list)
        return True

    def process_content(self, source_code_list: SourceCodeList):
        args = self.args
        content = source_code_list.to_str()

        if args.execute and self.llm and not args.human_as_model:
            content_length = self._get_content_length(content)
            if content_length > self.args.model_max_input_length:
                logger.warning(
                    f"Content(send to model) is {content_length} tokens, which is larger than the maximum input length {self.args.model_max_input_length}"
                )

        global_cancel.check_and_raise(token=self.args.cancel_token)

        if (
            (args.enable_auto_fix_merge or args.enable_auto_fix_lint)
            and args.execute
            and args.auto_merge == "editblock"
        ):
            code_merge_manager = CodeEditBlockManager(
                llm=self.llm, args=self.args, action=self
            )
            result = code_merge_manager.run(
                query=args.query, source_code_list=source_code_list
            )
            # 将对话添加到 conversation_manager
            self._add_conversations_to_manager(
                result, self.conversation_manager, self.namespace
            )
            return

        if args.execute:
            self.printer.print_in_terminal("code_generation_start")
            start_time = time.time()
            if args.auto_merge == "diff":
                generate = CodeAutoGenerateDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "strict_diff":
                generate = CodeAutoGenerateStrictDiff(
                    llm=self.llm, args=self.args, action=self
                )
            elif args.auto_merge == "editblock":
                generate = CodeAutoGenerateEditBlock(
                    llm=self.llm, args=self.args, action=self
                )
            else:
                generate = CodeAutoGenerate(llm=self.llm, args=self.args, action=self)

            generate_result = generate.single_round_run(
                query=args.query, source_code_list=source_code_list
            )

            elapsed_time = time.time() - start_time
            speed = (
                generate_result.metadata.get("generated_tokens_count", 0) / elapsed_time
                if elapsed_time > 0
                else 0
            )
            model_names = ",".join(get_llm_names(generate.llms))
            input_tokens_cost = generate_result.metadata.get("input_tokens_cost", 0)
            generated_tokens_cost = generate_result.metadata.get(
                "generated_tokens_cost", 0
            )
            self.printer.print_in_terminal(
                "code_generation_complete",
                duration=elapsed_time,
                input_tokens=generate_result.metadata.get("input_tokens_count", 0),
                output_tokens=generate_result.metadata.get("generated_tokens_count", 0),
                input_cost=input_tokens_cost,
                output_cost=generated_tokens_cost,
                speed=round(speed, 2),
                model_names=model_names,
                sampling_count=len(generate_result.contents),
            )

            get_event_manager(self.args.event_file).write_result(
                EventContentCreator.create_result(
                    content=EventContentCreator.ResultTokenStatContent(
                        model_name=model_names,
                        elapsed_time=elapsed_time,
                        input_tokens=generate_result.metadata.get(
                            "input_tokens_count", 0
                        ),
                        output_tokens=generate_result.metadata.get(
                            "generated_tokens_count", 0
                        ),
                        input_cost=input_tokens_cost,
                        output_cost=generated_tokens_cost,
                        speed=round(speed, 2),
                    )
                ).to_dict(),
                metadata=EventMetadata(action_file=self.args.file).to_dict(),
            )

            global_cancel.check_and_raise(token=self.args.cancel_token)

            merge_result = None
            if args.execute and args.auto_merge:
                self.printer.print_in_terminal("code_merge_start")
                if args.auto_merge == "diff":
                    code_merge = CodeAutoMergeDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "strict_diff":
                    code_merge = CodeAutoMergeStrictDiff(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                elif args.auto_merge == "editblock":
                    code_merge = CodeAutoMergeEditBlock(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )
                else:
                    code_merge = CodeAutoMerge(llm=self.llm, args=self.args)
                    merge_result = code_merge.merge_code(
                        generate_result=generate_result
                    )

            if merge_result is not None:
                content = merge_result.contents[0]
                store_code_model_conversation(
                    args=self.args,
                    instruction=self.args.query,
                    conversations=merge_result.conversations[0],
                    model=self.llm.default_model_name,
                )
            else:
                content = generate_result.contents[0]

                store_code_model_conversation(
                    args=self.args,
                    instruction=self.args.query,
                    conversations=generate_result.conversations[0],
                    model=self.llm.default_model_name,
                )



#!/usr/bin/env python3
"""
演示新的粘贴功能

这个脚本展示了如何使用新的粘贴处理功能:
1. Ctrl+P 粘贴多行内容并自动保存为文件
2. 占位符自动解析为实际内容
"""

import os
import tempfile
from prompt_toolkit import PromptSession
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.clipboard import ClipboardData
from autocoder.common.terminal_paste import register_paste_handler, resolve_paste_placeholders


def demo_paste_functionality():
    """演示粘贴功能的完整流程"""
    print("🎯 Terminal Paste 功能演示")
    print("=" * 50)
    
    # 创建键绑定
    kb = KeyBindings()
    register_paste_handler(kb)
    
    print("✅ 粘贴处理器已注册")
    print("📋 使用说明:")
    print("   - 复制多行文本到剪贴板")
    print("   - 按 Ctrl+P 粘贴 (多行内容会保存为文件)")
    print("   - 按 Ctrl+V 正常粘贴 (单行内容)")
    print("   - 输入 'quit' 退出演示")
    print()
    
    # 创建 PromptSession
    session = PromptSession(
        key_bindings=kb
        # 注意：bracketed paste 通过 Keys.BracketedPaste 绑定处理
    )
    
    while True:
        try:
            # 获取用户输入
            user_input = session.prompt("📝 请输入命令 (或按 Ctrl+P 粘贴): ")
            
            if user_input.lower() in ['quit', 'exit', 'q']:
                print("👋 演示结束")
                break
            
            # 解析占位符
            resolved_input = resolve_paste_placeholders(user_input)
            
            # 显示结果
            if resolved_input != user_input:
                print("🔄 占位符已解析:")
                print(f"   原始输入: {user_input}")
                print(f"   解析后: {resolved_input}")
            else:
                print(f"📤 您输入了: {resolved_input}")
            
            print()
            
        except KeyboardInterrupt:
            print("\n👋 演示结束")
            break
        except Exception as e:
            print(f"❌ 错误: {e}")


def create_sample_clipboard_content():
    """创建示例剪贴板内容用于测试"""
    sample_content = """def hello_world():
    print("Hello, World!")
    return "success"

if __name__ == "__main__":
    result = hello_world()
    print(f"Result: {result}")"""
    
    print("📋 示例多行内容:")
    print(sample_content)
    print()
    print("💡 提示: 复制上面的代码，然后在演示中按 Ctrl+P 粘贴")
    print()


if __name__ == "__main__":
    print("🚀 开始 Terminal Paste 功能演示")
    print()
    
    # 显示示例内容
    create_sample_clipboard_content()
    
    # 运行演示
    demo_paste_functionality() 


"""
PersistConversationManager 数据模型定义
"""

import time
import uuid
from typing import Union, Dict, List, Optional, Any
from dataclasses import dataclass, field
from .llm_stats_models import LLMCallMetadata


@dataclass
class ConversationMessage:
    """对话消息数据模型"""

    role: str
    content: Union[str, dict, list]
    timestamp: float = field(default_factory=time.time)
    message_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    metadata: Optional[dict] = None
    llm_metadata: Optional[LLMCallMetadata] = None
    reasoning_content: Optional[str] = None

    def __post_init__(self):
        """数据验证"""
        self._validate()

    def _validate(self):
        """验证消息数据"""
        # 验证角色
        valid_roles = ["system", "user", "assistant"]
        if not self.role or self.role not in valid_roles:
            raise ValueError(f"无效的消息角色: {self.role}，有效角色: {valid_roles}")

        # 验证内容
        if self.content is None or (
            isinstance(self.content, str) and len(self.content) == 0
        ):
            raise ValueError("消息内容不能为空")

        # 验证时间戳
        if not isinstance(self.timestamp, (int, float)) or self.timestamp <= 0:
            raise ValueError("无效的时间戳")

        # 验证消息ID
        if not isinstance(self.message_id, str) or len(self.message_id) == 0:
            raise ValueError("消息ID不能为空")

    def to_dict(self) -> dict:
        """序列化为字典"""
        result = {
            "role": self.role,
            "content": self.content,
            "timestamp": self.timestamp,
            "message_id": self.message_id,
            "metadata": self.metadata,
        }

        # 序列化 LLM 元数据
        if self.llm_metadata is not None:
            result["llm_metadata"] = self.llm_metadata.to_dict()

        # 序列化 reasoning_content（仅当有值时落盘）
        if self.reasoning_content is not None:
            result["reasoning_content"] = self.reasoning_content

        return result

    @classmethod
    def from_dict(cls, data: dict) -> "ConversationMessage":
        """从字典反序列化"""
        # 处理 LLM 元数据
        llm_metadata = None
        if "llm_metadata" in data and data["llm_metadata"] is not None:
            llm_metadata = LLMCallMetadata.from_dict(data["llm_metadata"])

        # 处理 reasoning_content：优先顶层字段，其次兼容旧数据 metadata 中的 reasoning_content
        reasoning_content = data.get("reasoning_content")
        if reasoning_content is None:
            msg_metadata = data.get("metadata")
            if isinstance(msg_metadata, dict):
                reasoning_content = msg_metadata.get("reasoning_content")

        return cls(
            role=data["role"],
            content=data["content"],
            timestamp=data["timestamp"],
            message_id=data["message_id"],
            metadata=data.get("metadata"),
            llm_metadata=llm_metadata,
            reasoning_content=reasoning_content,
        )


@dataclass
class Conversation:
    """对话数据模型"""

    name: str
    conversation_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    description: Optional[str] = None
    created_at: float = field(default_factory=time.time)
    updated_at: float = field(default_factory=time.time)
    messages: List[dict] = field(default_factory=list)
    metadata: Optional[dict] = None
    version: int = 1

    def __post_init__(self):
        """数据验证"""
        self._validate()

    def _validate(self):
        """验证对话数据"""
        # 验证名称
        if not self.name or (isinstance(self.name, str) and len(self.name) == 0):
            raise ValueError("对话名称不能为空")

        # 验证版本号
        if not isinstance(self.version, int) or self.version <= 0:
            raise ValueError("版本号必须是正整数")

        # 验证时间戳
        if not isinstance(self.created_at, (int, float)) or self.created_at <= 0:
            raise ValueError("无效的创建时间戳")

        if (
            not isinstance(self.updated_at, (int, float))
            or self.updated_at < self.created_at
        ):
            raise ValueError("无效的更新时间戳")

        # 验证对话ID
        if not isinstance(self.conversation_id, str) or len(self.conversation_id) == 0:
            raise ValueError("对话ID不能为空")

        # 验证消息列表
        if not isinstance(self.messages, list):
            raise ValueError("消息列表必须是列表类型")

    def add_message(self, message: ConversationMessage):
        """添加消息到对话"""
        self.messages.append(message.to_dict())
        self.updated_at = time.time()

    def remove_message(self, message_id: str) -> bool:
        """从对话中删除消息"""
        for i, msg in enumerate(self.messages):
            if msg.get("message_id") == message_id:
                del self.messages[i]
                self.updated_at = time.time()
                return True
        return False

    def get_message(self, message_id: str) -> Optional[dict]:
        """从对话中获取消息"""
        for msg in self.messages:
            if msg.get("message_id") == message_id:
                return msg
        return None

    def to_dict(self) -> dict:
        """序列化为字典"""
        return {
            "conversation_id": self.conversation_id,
            "name": self.name,
            "description": self.description,
            "created_at": self.created_at,
            "updated_at": self.updated_at,
            "messages": self.messages,
            "metadata": self.metadata,
            "version": self.version,
        }

    @classmethod
    def from_dict(cls, data: dict) -> "Conversation":
        """从字典反序列化"""
        return cls(
            conversation_id=data["conversation_id"],
            name=data["name"],
            description=data.get("description"),
            created_at=data["created_at"],
            updated_at=data["updated_at"],
            messages=data.get("messages", []),
            metadata=data.get("metadata"),
            version=data.get("version", 1),
        )



"""
WebSearchTool Module

This module implements the WebSearchTool and WebSearchToolResolver classes
for providing web search functionality based on Firecrawl, Metaso, or BochaAI
within the BaseAgent framework.
"""

import os
import traceback
import json
from typing import Dict, Any, List, Optional

import byzerllm
from loguru import logger
from numpy import log

from autocoder.agent.base_agentic.types import BaseTool, ToolResult
from autocoder.agent.base_agentic.tool_registry import ToolRegistry
from autocoder.agent.base_agentic.tools.base_tool_resolver import BaseToolResolver
from autocoder.agent.base_agentic.types import ToolDescription, ToolExample
from autocoder.rag.tools.metaso_sdk import MetasoClient
from autocoder.rag.tools.bochaai_sdk import BochaAIClient


class WebSearchTool(BaseTool):
    """Web search tool using Firecrawl, Metaso, or BochaAI for web search"""
    query: str  # Search query
    limit: Optional[int] = 5  # Return result count limit
    sources: Optional[str] = None  # Search source types, comma-separated: web,news,images
    scrape_options: Optional[str] = None  # Scraping options, JSON string format
    location: Optional[str] = None  # Search location
    tbs: Optional[str] = None  # Time filter parameters


class WebSearchToolResolver(BaseToolResolver):
    """Web search tool resolver that implements search logic"""

    def __init__(self, agent, tool, args):
        super().__init__(agent, tool, args)
        self.tool: WebSearchTool = tool

    def _search_with_bochaai(self) -> ToolResult:
        """Search using BochaAI"""
        try:
            # Check if BochaAI API key is provided
            api_key = getattr(self.args, 'bochaai_api_key', None) or os.getenv('BOCHAAI_API_KEY')
            if not api_key:
                return ToolResult(
                    success=False,
                    message="BochaAI API key not provided, please set --bochaai_api_key parameter or BOCHAAI_API_KEY environment variable",
                    content=[]
                )

            # Initialize BochaAI client
            client = BochaAIClient(api_key=api_key)

            # Prepare search parameters
            freshness = "noLimit"  # Default no time limit
            if self.tool.tbs:
                # Map time filter parameters
                tbs_mapping = {
                    "d": "oneDay",
                    "w": "oneWeek",
                    "m": "oneMonth",
                    "y": "oneYear"
                }
                freshness = tbs_mapping.get(self.tool.tbs, "noLimit")

            # Parse scrape_options to get additional parameters
            include_summary = False
            include_sites = None
            exclude_sites = None
            if self.tool.scrape_options:
                try:
                    scrape_opts = json.loads(self.tool.scrape_options)
                    include_summary = scrape_opts.get('include_summary', False) or scrape_opts.get('summary', False)
                    include_sites = scrape_opts.get('include_sites')
                    exclude_sites = scrape_opts.get('exclude_sites')
                except json.JSONDecodeError:
                    pass

            # Execute search
            logger.info(f"Performing web search using BochaAI, query: {self.tool.query}")
            result = client.search(
                query=self.tool.query,
                count=self.tool.limit,
                freshness=freshness,
                summary=include_summary,
                include=include_sites,
                exclude=exclude_sites
            )

            if not result.success:
                return ToolResult(
                    success=False,
                    message=f"BochaAI search failed: {result.error}",
                    content=[]
                )

            # Format search results
            search_results = []

            # Process web results
            for webpage in result.webpages:
                result_item = {
                    "type": "web",
                    "title": webpage.name,
                    "url": webpage.url,
                    "description": webpage.snippet,
                    "provider": "bochaai"
                }

                if webpage.summary:
                    result_item["summary"] = webpage.summary
                if webpage.date_published:
                    result_item["date"] = webpage.date_published
                if webpage.site_name:
                    result_item["site_name"] = webpage.site_name
                if webpage.site_icon:
                    result_item["site_icon"] = webpage.site_icon

                search_results.append(result_item)

            # Process image results (if needed)
            if self.tool.sources and "images" in self.tool.sources:
                for image in result.images:
                    image_item = {
                        "type": "image",
                        "url": image.content_url,
                        "thumbnail_url": image.thumbnail_url,
                        "host_page_url": image.host_page_url,
                        "width": image.width,
                        "height": image.height,
                        "provider": "bochaai"
                    }
                    if image.name:
                        image_item["title"] = image.name
                    search_results.append(image_item)

            return ToolResult(
                success=True,
                message=f"Successfully found {len(search_results)} results (using BochaAI)",
                content=search_results
            )

        except Exception as e:
            logger.error(f"BochaAI search failed: {str(e)}")
            return ToolResult(
                success=False,
                message=f"BochaAI search failed: {str(e)}",
                content=traceback.format_exc()
            )

    def _search_with_metaso(self) -> ToolResult:
        """Search using Metaso"""
        try:
            # Check if Metaso API key is provided
            api_key = getattr(self.args, 'metaso_api_key', None) or os.getenv('METASO_API_KEY')
            if not api_key:
                return ToolResult(
                    success=False,
                    message="Metaso API key not provided, please set --metaso_api_key parameter or METASO_API_KEY environment variable",
                    content=[]
                )

            # Initialize Metaso client
            client = MetasoClient(api_key=api_key)

            # Prepare search parameters
            scope = "webpage"  # Default search web pages
            if self.tool.sources:
                # Map sources to Metaso's scope
                sources_list = [s.strip().lower() for s in self.tool.sources.split(',')]
                if "news" in sources_list:
                    scope = "news"
                # Metaso does not support images search yet, keep default webpage

            # Parse scrape_options to get additional parameters
            include_summary = False
            include_raw_content = False
            if self.tool.scrape_options:
                try:
                    scrape_opts = json.loads(self.tool.scrape_options)
                    include_summary = scrape_opts.get('include_summary', False)
                    include_raw_content = scrape_opts.get('include_raw_content', False)
                except json.JSONDecodeError:
                    pass

            # Execute search
            logger.info(f"Performing web search using Metaso, query: {self.tool.query}")
            result = client.search(
                query=self.tool.query,
                scope=scope,
                size=self.tool.limit,
                include_summary=include_summary,
                include_raw_content=include_raw_content
            )

            if not result.success:
                return ToolResult(
                    success=False,
                    message=f"Metaso search failed: {result.error}",
                    content=[]
                )

            # Format search results
            search_results = []
            for webpage in result.webpages:
                result_item = {
                    "type": "web",
                    "title": webpage.title,
                    "url": webpage.link,
                    "description": webpage.snippet,
                    "position": webpage.position,
                    "score": webpage.score,
                    "provider": "metaso"
                }

                if webpage.date:
                    result_item["date"] = webpage.date
                if webpage.authors:
                    result_item["authors"] = webpage.authors

                search_results.append(result_item)

            return ToolResult(
                success=True,
                message=f"Successfully found {len(search_results)} results (using Metaso)",
                content=search_results
            )

        except Exception as e:
            logger.error(f"Metaso search failed: {str(e)}")
            return ToolResult(
                success=False,
                message=f"Metaso search failed: {str(e)}",
                content=traceback.format_exc()
            )

    def _search_with_firecrawl(self) -> ToolResult:
        """Search using Firecrawl"""
        try:
            # Check if Firecrawl API key is provided
            api_key = getattr(self.args, 'firecrawl_api_key', None) or os.getenv('FIRECRAWL_API_KEY')
            if not api_key:
                return ToolResult(
                    success=False,
                    message="Firecrawl API key not provided, please set --firecrawl_api_key parameter or FIRECRAWL_API_KEY environment variable",
                    content=[]
                )

            # Import Firecrawl SDK
            try:
                from firecrawl import Firecrawl
            except ImportError:
                return ToolResult(
                    success=False,
                    message="Firecrawl SDK not installed, please run: pip install firecrawl-py",
                    content=[]
                )

            # Initialize Firecrawl client
            firecrawl = Firecrawl(api_key=api_key)

            # Prepare search parameters
            search_params = {
                "query": self.tool.query,
                "limit": self.tool.limit
            }

            # Add optional parameters
            if self.tool.sources:
                # Parse comma-separated sources string
                sources_list = [s.strip() for s in self.tool.sources.split(',') if s.strip()]
                search_params["sources"] = sources_list
            if self.tool.scrape_options:
                # Parse JSON format scrape_options string
                try:
                    scrape_options_dict = json.loads(self.tool.scrape_options)
                    search_params["scrape_options"] = scrape_options_dict
                except json.JSONDecodeError as e:
                    logger.warning(f"scrape_options JSON parsing failed: {e}, ignoring this parameter")
            if self.tool.location:
                search_params["location"] = self.tool.location
            if self.tool.tbs:
                search_params["tbs"] = self.tool.tbs

            # Execute search
            logger.info(f"Starting web search, query: {self.tool.query}")
            results = firecrawl.search(**search_params)

            # Check result type and success status
            if not results:
                return ToolResult(
                    success=False,
                    message="No results returned",
                    content=[]
                )

            # New version firecrawl returns SearchData object with different structure
            if hasattr(results, 'success') and not results.success:
                error_msg = getattr(results, 'error', 'Unknown error')
                return ToolResult(
                    success=False,
                    message=f"Search failed: {error_msg}",
                    content=[]
                )

            # Format search results
            search_results = []
            # New version firecrawl returned data might be attribute instead of dict key
            if hasattr(results, 'data'):
                data = results.data
            else:
                data = results.get('data', {}) if hasattr(results, 'get') else {}

            # If data is empty, log information for debugging
            if not data or (isinstance(data, dict) and not data):
                logger.info("Search returned empty data, possibly due to API quota limit or service status issues")

            # Process web results - new version might be directly in data, not in data.web
            web_results = []
            if hasattr(data, 'web'):
                web_results = data.web
            elif isinstance(data, dict) and 'web' in data:
                web_results = data['web']
            elif isinstance(data, list):
                # Might directly return result list
                web_results = data

            if web_results:
                for item in web_results:
                    # Handle possible object or dictionary format
                    title = getattr(item, 'title', None) or (item.get('title', '') if hasattr(item, 'get') else '')
                    url = getattr(item, 'url', None) or (item.get('url', '') if hasattr(item, 'get') else '')
                    description = getattr(item, 'description', None) or (item.get('description', '') if hasattr(item, 'get') else '')
                    position = getattr(item, 'position', None) or (item.get('position', 0) if hasattr(item, 'get') else 0)

                    result_item = {
                        "type": "web",
                        "title": title,
                        "url": url,
                        "description": description,
                        "position": position
                    }

                    # If there is scraped content, add to results
                    markdown = getattr(item, 'markdown', None) or (item.get('markdown') if hasattr(item, 'get') else None)
                    if markdown:
                        result_item['content'] = markdown

                    links = getattr(item, 'links', None) or (item.get('links') if hasattr(item, 'get') else None)
                    if links:
                        result_item['links'] = links

                    metadata = getattr(item, 'metadata', None) or (item.get('metadata') if hasattr(item, 'get') else None)
                    if metadata:
                        result_item['metadata'] = metadata

                    search_results.append(result_item)

            # Process news results
            if 'news' in data:
                for item in data['news']:
                    result_item = {
                        "type": "news",
                        "title": item.get('title', ''),
                        "url": item.get('url', ''),
                        "snippet": item.get('snippet', ''),
                        "date": item.get('date', ''),
                        "position": item.get('position', 0)
                    }
                    search_results.append(result_item)

            # Process image results
            if 'images' in data:
                for item in data['images']:
                    result_item = {
                        "type": "image",
                        "title": item.get('title', ''),
                        "imageUrl": item.get('imageUrl', ''),
                        "url": item.get('url', ''),
                        "imageWidth": item.get('imageWidth', 0),
                        "imageHeight": item.get('imageHeight', 0),
                        "position": item.get('position', 0)
                    }
                    search_results.append(result_item)

            return ToolResult(
                success=True,
                message=f"Successfully found {len(search_results)} results (using Firecrawl)",
                content=search_results
            )

        except Exception as e:
            logger.error(f"Firecrawl search failed: {str(e)}")
            return ToolResult(
                success=False,
                message=f"Firecrawl search failed: {str(e)}",
                content=traceback.format_exc()
            )

    def resolve(self) -> ToolResult:
        """Implement web search tool resolution logic"""
        try:
            # Determine which provider to use
            arg_bochaai_key = self.args.bochaai_api_key
            bochaai_key = arg_bochaai_key

            arg_metaso_key = self.args.metaso_api_key
            metaso_key = arg_metaso_key

            arg_firecrawl_key = self.args.firecrawl_api_key
            firecrawl_key = arg_firecrawl_key

            # Check if any API key is configured
            if not any([bochaai_key, metaso_key, firecrawl_key]):
                # No API key configured, guide to use search engines and curl
                search_query = self.tool.query.replace(' ', '+')

                # Build search suggestions
                search_suggestions = []
                search_suggestions.append(f"# Use search engines for web search")
                search_suggestions.append(f"# 1. Google Search:")
                search_suggestions.append(f"#    Visit: https://www.google.com/search?q={search_query}")
                search_suggestions.append(f"")
                search_suggestions.append(f"# 2. DuckDuckGo Search:")
                search_suggestions.append(f"#    Visit: https://duckduckgo.com/?q={search_query}")
                search_suggestions.append(f"")
                search_suggestions.append(f"# 3. Use curl to get search result pages (need to handle anti-crawling mechanisms):")
                search_suggestions.append(f"curl -s -L -H 'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36' 'https://www.google.com/search?q={search_query}'")
                search_suggestions.append(f"")
                search_suggestions.append(f"# 4. If you know specific websites, search them directly:")
                search_suggestions.append(f"curl -s -L 'https://example.com/search?q={search_query}'")

                if self.tool.sources:
                    search_suggestions.append(f"")
                    search_suggestions.append(f"# Specified search type: {self.tool.sources}")
                    if "news" in self.tool.sources:
                        search_suggestions.append(f"# For news search, visit: https://news.google.com/search?q={search_query}")
                    if "images" in self.tool.sources:
                        search_suggestions.append(f"# For image search, visit: https://www.google.com/search?tbm=isch&q={search_query}")

                if self.tool.location:
                    search_suggestions.append(f"")
                    search_suggestions.append(f"# Geographic location restriction: {self.tool.location}")
                    search_suggestions.append(f"# You can add location keywords to search")

                if self.tool.tbs:
                    search_suggestions.append(f"")
                    search_suggestions.append(f"# Time filter: {self.tool.tbs}")
                    search_suggestions.append(f"# Google search can use time filter options in the tools menu")

                suggestion_text = "\n".join(search_suggestions)

                return ToolResult(
                    success=False,
                    message=f"No web search API key configured (BochaAI, Metaso, Firecrawl).",
                    content={}
                )

            # Determine provider priority: BochaAI > Metaso > Firecrawl
            provider = None
            selected_key_name = None
            selected_key_value = None
            if bochaai_key:
                provider = "bochaai"
                selected_key_name = "bochaai_api_key" if arg_bochaai_key else "BOCHAAI_API_KEY"
                selected_key_value = bochaai_key
            elif metaso_key:
                provider = "metaso"
                selected_key_name = "metaso_api_key" if arg_metaso_key else "METASO_API_KEY"
                selected_key_value = metaso_key
            elif firecrawl_key:
                provider = "firecrawl"
                selected_key_name = "firecrawl_api_key" if arg_firecrawl_key else "FIRECRAWL_API_KEY"
                selected_key_value = firecrawl_key

            key_prefix = (selected_key_value[:10] if selected_key_value else "")
            logger.info(f"Using search provider: {provider}, key_name: {selected_key_name}, key_prefix: {key_prefix}")

            if provider == "bochaai":
                return self._search_with_bochaai()
            elif provider == "metaso":
                return self._search_with_metaso()
            elif provider == "firecrawl":
                return self._search_with_firecrawl()

            # This situation should not happen theoretically
            return ToolResult(
                success=False,
                message="Internal error: Unable to determine search provider",
                content=[]
            )

        except Exception as e:
            logger.error(f"Web search tool execution failed: {str(e)}")
            return ToolResult(
                success=False,
                message=f"Web search tool execution failed: {str(e)}",
                content=traceback.format_exc()
            )


class WebSearchToolDescGenerator:
    def __init__(self, params: Dict[str, Any]):
        self.params = params

    @byzerllm.prompt()
    def web_search_description(self) -> Dict:
        """
        Description: Request to perform web search using Firecrawl, Metaso or BochaAI API. Use this when you need to search for current web content, news, or images to gather information for completing your task. This tool supports multiple search sources and can optionally scrape the full content of search result pages. You can use web_crawl to fetch the url of the search result pages.
        Parameters:
        - query: (required) The search query string to search for
        - limit: (optional) Maximum number of results to return (default: 5)
        - sources: (optional) Comma-separated search source types, e.g., "web", "news", "images", "web,news"
        - scrape_options: (optional) Additional scraping options as JSON string, e.g., '{"formats": ["markdown"], "include_summary": true, "include_sites": "example.com", "exclude_sites": "bad.com"}'
        - location: (optional) Geographic location to focus the search (Firecrawl only)
        - tbs: (optional) Time-based search parameters for filtering results (Firecrawl/BochaAI)
        Usage:
        <web_search>
        <query>Your search query here</query>
        <limit>5</limit>
        <sources>web</sources>
        </web_search>
        """
        return self.params


def register_web_search_tool():
    """Register web search tool"""
    desc_gen = WebSearchToolDescGenerator({})

    # 准备工具描述
    description = ToolDescription(
        description=desc_gen.web_search_description.prompt()
    )

    # 准备工具示例
    example = ToolExample(
        title="Web search tool usage example",
        body="""<web_search>
<query>Python RAG system implementation</query>
<limit>3</limit>
<sources>web</sources>
</web_search>"""
    )

    # 注册工具
    ToolRegistry.register_tool(
        tool_tag="web_search",  # XML标签名
        tool_cls=WebSearchTool,  # 工具类
        resolver_cls=WebSearchToolResolver,  # 解析器类
        description=description,  # 工具描述
        example=example,  # 工具示例
        use_guideline="Use this tool to search for web content across different sources (web pages, news, images). It can optionally scrape complete content from search results. Ideal for gathering current information from the internet to help complete tasks that require up-to-date data."  # 使用指南
    )
